from typing import overload
from enum import IntEnum
import abc
import datetime
import typing

import QuantConnect
import QuantConnect.Data
import QuantConnect.Data.Auxiliary
import QuantConnect.Data.Consolidators
import QuantConnect.Data.Market
import QuantConnect.Data.UniverseSelection
import QuantConnect.Interfaces
import QuantConnect.Lean.Engine.DataFeeds
import QuantConnect.Lean.Engine.DataFeeds.Enumerators
import QuantConnect.Lean.Engine.Results
import QuantConnect.Python
import QuantConnect.Securities
import QuantConnect.Util
import System
import System.Collections
import System.Collections.Generic

QuantConnect_Lean_Engine_DataFeeds_Enumerators_EnqueueableEnumerator_T = typing.TypeVar("QuantConnect_Lean_Engine_DataFeeds_Enumerators_EnqueueableEnumerator_T")
QuantConnect_Lean_Engine_DataFeeds_Enumerators_FilterEnumerator_T = typing.TypeVar("QuantConnect_Lean_Engine_DataFeeds_Enumerators_FilterEnumerator_T")
QuantConnect_Lean_Engine_DataFeeds_Enumerators_SortEnumerator_TKey = typing.TypeVar("QuantConnect_Lean_Engine_DataFeeds_Enumerators_SortEnumerator_TKey")
QuantConnect_Lean_Engine_DataFeeds_Enumerators_RefreshEnumerator_T = typing.TypeVar("QuantConnect_Lean_Engine_DataFeeds_Enumerators_RefreshEnumerator_T")
QuantConnect_Lean_Engine_DataFeeds_Enumerators_SynchronizingEnumerator_T = typing.TypeVar("QuantConnect_Lean_Engine_DataFeeds_Enumerators_SynchronizingEnumerator_T")
QuantConnect_Lean_Engine_DataFeeds_Enumerators_RateLimitEnumerator_T = typing.TypeVar("QuantConnect_Lean_Engine_DataFeeds_Enumerators_RateLimitEnumerator_T")
QuantConnect_Lean_Engine_DataFeeds_Enumerators_ScannableEnumerator_T = typing.TypeVar("QuantConnect_Lean_Engine_DataFeeds_Enumerators_ScannableEnumerator_T")
QuantConnect_Lean_Engine_DataFeeds_Enumerators__EventContainer_Callable = typing.TypeVar("QuantConnect_Lean_Engine_DataFeeds_Enumerators__EventContainer_Callable")
QuantConnect_Lean_Engine_DataFeeds_Enumerators__EventContainer_ReturnType = typing.TypeVar("QuantConnect_Lean_Engine_DataFeeds_Enumerators__EventContainer_ReturnType")


class EnqueueableEnumerator(typing.Generic[QuantConnect_Lean_Engine_DataFeeds_Enumerators_EnqueueableEnumerator_T], System.Object, System.Collections.Generic.IEnumerator[QuantConnect_Lean_Engine_DataFeeds_Enumerators_EnqueueableEnumerator_T]):
    """
    An implementation of IEnumerator{T} that relies on the
    enqueue method being called and only ends when stop
    is called
    """

    @property
    def count(self) -> int:
        """Gets the current number of items held in the internal queue"""
        ...

    @property
    def has_finished(self) -> bool:
        """Returns true if the enumerator has finished and will not accept any more data"""
        ...

    @property
    def current(self) -> QuantConnect_Lean_Engine_DataFeeds_Enumerators_EnqueueableEnumerator_T:
        """Gets the element in the collection at the current position of the enumerator."""
        ...

    def __init__(self, blocking: bool = False) -> None:
        """
        Initializes a new instance of the EnqueueableEnumerator{T} class
        
        :param blocking: Specifies whether or not to use the blocking behavior
        """
        ...

    def dispose(self) -> None:
        """Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources."""
        ...

    def enqueue(self, data: QuantConnect_Lean_Engine_DataFeeds_Enumerators_EnqueueableEnumerator_T) -> None:
        """
        Enqueues the new data into this enumerator
        
        :param data: The data to be enqueued
        """
        ...

    def move_next(self) -> bool:
        """
        Advances the enumerator to the next element of the collection.
        
        :returns: true if the enumerator was successfully advanced to the next element; false if the enumerator has passed the end of the collection.
        """
        ...

    def reset(self) -> None:
        """Sets the enumerator to its initial position, which is before the first element in the collection."""
        ...

    def stop(self) -> None:
        """
        Signals the enumerator to stop enumerating when the items currently
        held inside are gone. No more items will be added to this enumerator.
        """
        ...


class FilterEnumerator(typing.Generic[QuantConnect_Lean_Engine_DataFeeds_Enumerators_FilterEnumerator_T], System.Object, System.Collections.Generic.IEnumerator[QuantConnect_Lean_Engine_DataFeeds_Enumerators_FilterEnumerator_T]):
    """Enumerator that allow applying a filtering function"""

    @property
    def current(self) -> QuantConnect_Lean_Engine_DataFeeds_Enumerators_FilterEnumerator_T:
        """Gets the current item in the FilterEnumerator"""
        ...

    def __init__(self, enumerator: System.Collections.Generic.IEnumerator[QuantConnect_Lean_Engine_DataFeeds_Enumerators_FilterEnumerator_T], filter: typing.Callable[[QuantConnect_Lean_Engine_DataFeeds_Enumerators_FilterEnumerator_T], bool]) -> None:
        """
        Creates a new instance
        
        :param enumerator: The underlying enumerator to filter on
        :param filter: The filter to apply
        """
        ...

    def dispose(self) -> None:
        """Disposes the FilterEnumerator"""
        ...

    def move_next(self) -> bool:
        """Moves the FilterEnumerator to the next item"""
        ...

    def reset(self) -> None:
        """Resets the FilterEnumerator"""
        ...


class SynchronizingBaseDataEnumerator(QuantConnect.Lean.Engine.DataFeeds.Enumerators.SynchronizingEnumerator[QuantConnect.Data.BaseData]):
    """
    Represents an enumerator capable of synchronizing other base data enumerators in time.
    This assumes that all enumerators have data time stamped in the same time zone
    """

    @overload
    def __init__(self, *enumerators: typing.Union[System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData], typing.Iterable[System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData]]]) -> None:
        """
        Initializes a new instance of the SynchronizingBaseDataEnumerator class
        
        :param enumerators: The enumerators to be synchronized. NOTE: Assumes the same time zone for all data
        """
        ...

    @overload
    def __init__(self, enumerators: typing.List[System.Collections.IEnumerator]) -> None:
        """
        Initializes a new instance of the SynchronizingBaseDataEnumerator class
        
        :param enumerators: The enumerators to be synchronized. NOTE: Assumes the same time zone for all data
        """
        ...

    def get_instance_time(self, instance: QuantConnect.Data.BaseData) -> datetime.datetime:
        """
        Gets the Timestamp for the data
        
        
        This codeEntityType is protected.
        """
        ...


class ITradableDatesNotifier(metaclass=abc.ABCMeta):
    """
    Interface which will provide an event handler
    who will be fired with each new tradable day
    """

    @property
    @abc.abstractmethod
    def new_tradable_date(self) -> _EventContainer[typing.Callable[[System.Object, QuantConnect.NewTradableDateEventArgs], typing.Any], typing.Any]:
        """Event fired when there is a new tradable date"""
        ...

    @new_tradable_date.setter
    def new_tradable_date(self, value: _EventContainer[typing.Callable[[System.Object, QuantConnect.NewTradableDateEventArgs], typing.Any], typing.Any]) -> None:
        ...


class StrictDailyEndTimesEnumerator(System.Object, System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData]):
    """Enumerator that will handle adjusting daily strict end times if appropriate"""

    @property
    def current(self) -> QuantConnect.Data.BaseData:
        """Current value of the enumerator"""
        ...

    def __init__(self, underlying: System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData], security_exchange_hours: QuantConnect.Securities.SecurityExchangeHours, local_start_time: typing.Union[datetime.datetime, datetime.date]) -> None:
        """Creates a new instance"""
        ...

    def dispose(self) -> None:
        """Dispose the enumerator"""
        ...

    def move_next(self) -> bool:
        """Move to the next date"""
        ...

    def reset(self) -> None:
        """Reset the enumerator"""
        ...


class QuoteBarFillForwardEnumerator(System.Object, System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData]):
    """
    The QuoteBarFillForwardEnumerator wraps an existing base data enumerator
    If the current QuoteBar has null Bid and/or Ask bars, it copies them from the previous QuoteBar
    """

    @property
    def current(self) -> QuantConnect.Data.BaseData:
        """Gets the element in the collection at the current position of the enumerator."""
        ...

    def __init__(self, enumerator: System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData]) -> None:
        """Initializes a new instance of the FillForwardEnumerator class"""
        ...

    def dispose(self) -> None:
        """Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources."""
        ...

    def move_next(self) -> bool:
        """
        Advances the enumerator to the next element of the collection.
        
        :returns: true if the enumerator was successfully advanced to the next element; false if the enumerator has passed the end of the collection.
        """
        ...

    def reset(self) -> None:
        """Sets the enumerator to its initial position, which is before the first element in the collection."""
        ...


class SortEnumerator(typing.Generic[QuantConnect_Lean_Engine_DataFeeds_Enumerators_SortEnumerator_TKey], System.Object, System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData]):
    """
    Provides an enumerator for sorting collections of BaseData objects based on a specified property.
    The sorting occurs lazily, only when enumeration begins.
    """

    @property
    def current(self) -> QuantConnect.Data.BaseData:
        """Gets the current BaseData element in the collection."""
        ...

    def __init__(self, data: typing.List[QuantConnect.Data.BaseData], key_selector: typing.Callable[[QuantConnect.Data.BaseData], QuantConnect_Lean_Engine_DataFeeds_Enumerators_SortEnumerator_TKey] = None) -> None:
        """
        Initializes a new instance of the SortEnumerator{TKey} class.
        
        :param data: The collection of BaseData to enumerate over.
        :param key_selector: A function that defines the key to sort by. Defaults to sorting by BaseData.end_time.
        """
        ...

    def dispose(self) -> None:
        """Releases all resources used by the SortEnumerator{TKey} and suppresses finalization."""
        ...

    def move_next(self) -> bool:
        """
        Advances the enumerator to the next element of the collection.
        
        :returns: true if the enumerator was successfully advanced to the next element;
        false if the enumerator has passed the end of the collection.
        """
        ...

    def reset(self) -> None:
        """Resets the enumerator to its initial position, which is before the first element in the collection."""
        ...

    @staticmethod
    def try_wrap_sort_enumerator(pre_sorted: bool, data: typing.List[QuantConnect.Data.BaseData]) -> System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData]:
        """
        Static method to wrap an enumerable with the sort enumerator.
        
        :param pre_sorted: Indicates if the data is pre-sorted.
        :param data: The data to be wrapped into the enumerator.
        :returns: An enumerator over the BaseData.
        """
        ...


class ConcatEnumerator(System.Object, System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData]):
    """Enumerator that will concatenate enumerators together sequentially enumerating them in the provided order"""

    @property
    def current(self) -> QuantConnect.Data.BaseData:
        """The current BaseData object"""
        ...

    @current.setter
    def current(self, value: QuantConnect.Data.BaseData) -> None:
        ...

    @property
    def can_emit_null(self) -> bool:
        """True if emitting a null data point is expected"""
        ...

    @can_emit_null.setter
    def can_emit_null(self, value: bool) -> None:
        ...

    def __init__(self, skip_duplicate_end_times: bool, *enumerators: typing.Union[System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData], typing.Iterable[System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData]]]) -> None:
        """
        Creates a new instance
        
        :param skip_duplicate_end_times: True will skip data points from enumerators if before or at the last end time
        :param enumerators: The sequence of enumerators to concatenate. Note that the order here matters, it will consume enumerators
        and dispose of them, even if they return true and their current is null, except for the last which will be kept!
        """
        ...

    def dispose(self) -> None:
        """Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources."""
        ...

    def move_next(self) -> bool:
        """
        Advances the enumerator to the next element of the collection.
        
        :returns: True if the enumerator was successfully advanced to the next element; false if the enumerator has passed the end of the collection.
        """
        ...

    def reset(self) -> None:
        """Sets the enumerator to its initial position, which is before the first element in the collection."""
        ...


class LastPointTracker(System.Object):
    """Tracks the last data point received by an enumerator."""

    @property
    def last_data_point(self) -> QuantConnect.Data.BaseData:
        """Tracks the last data point received by the enumerator."""
        ...

    @last_data_point.setter
    def last_data_point(self, value: QuantConnect.Data.BaseData) -> None:
        ...


class FillForwardEnumerator(System.Object, System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData]):
    """
    The FillForwardEnumerator wraps an existing base data enumerator and inserts extra 'base data' instances
    on a specified fill forward resolution
    """

    @property
    def use_strict_end_time(self) -> bool:
        """
        Whether to use strict daily end times
        
        
        This codeEntityType is protected.
        """
        ...

    @property
    def exchange(self) -> QuantConnect.Securities.SecurityExchange:
        """
        The exchange used to determine when to insert fill forward data
        
        
        This codeEntityType is protected.
        """
        ...

    @property
    def current(self) -> QuantConnect.Data.BaseData:
        """Gets the element in the collection at the current position of the enumerator."""
        ...

    def __init__(self, enumerator: System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData], exchange: QuantConnect.Securities.SecurityExchange, fill_forward_resolution: QuantConnect.Util.IReadOnlyRef[datetime.timedelta], is_extended_market_hours: bool, subscription_start_time: typing.Union[datetime.datetime, datetime.date], subscription_end_time: typing.Union[datetime.datetime, datetime.date], data_resolution: datetime.timedelta, data_time_zone: typing.Any, daily_strict_end_time_enabled: bool, data_type: typing.Type = None, last_point_tracker: QuantConnect.Lean.Engine.DataFeeds.Enumerators.LastPointTracker = None) -> None:
        """
        Initializes a new instance of the FillForwardEnumerator class that accepts
        a reference to the fill forward resolution, useful if the fill forward resolution is dynamic
        and changing as the enumeration progresses
        
        :param enumerator: The source enumerator to be filled forward
        :param exchange: The exchange used to determine when to insert fill forward data
        :param fill_forward_resolution: The resolution we'd like to receive data on
        :param is_extended_market_hours: True to use the exchange's extended market hours, false to use the regular market hours
        :param subscription_start_time: The start time of the subscription
        :param subscription_end_time: The end time of the subscription, once passing this date the enumerator will stop
        :param data_resolution: The source enumerator's data resolution
        :param data_time_zone: The time zone of the underlying source data. This is used for rounding calculations and
        is NOT the time zone on the BaseData instances (unless of course data time zone equals the exchange time zone)
        :param daily_strict_end_time_enabled: True if daily strict end times are enabled
        :param data_type: The configuration data type this enumerator is for
        :param last_point_tracker: A reference to the last point emitted before this enumerator is first enumerated
        """
        ...

    def dispose(self) -> None:
        """Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources."""
        ...

    def move_next(self) -> bool:
        """
        Advances the enumerator to the next element of the collection.
        
        :returns: true if the enumerator was successfully advanced to the next element; false if the enumerator has passed the end of the collection.
        """
        ...

    def requires_fill_forward_data(self, fill_forward_resolution: datetime.timedelta, previous: QuantConnect.Data.BaseData, next: QuantConnect.Data.BaseData, fill_forward: typing.Optional[QuantConnect.Data.BaseData]) -> typing.Tuple[bool, QuantConnect.Data.BaseData]:
        """
        Determines whether or not fill forward is required, and if true, will produce the new fill forward data
        
        
        This codeEntityType is protected.
        
        :param fill_forward_resolution: 
        :param previous: The last piece of data emitted by this enumerator
        :param next: The next piece of data on the source enumerator
        :param fill_forward: When this function returns true, this will have a non-null value, null when the function returns false
        :returns: True when a new fill forward piece of data was produced and should be emitted by this enumerator.
        """
        ...

    def reset(self) -> None:
        """Sets the enumerator to its initial position, which is before the first element in the collection."""
        ...


class LiveAuxiliaryDataSynchronizingEnumerator(System.Object, System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData]):
    """
    Represents an enumerator capable of synchronizing live equity data enumerators in time.
    This assumes that all enumerators have data time stamped in the same time zone.
    """

    @property
    def current(self) -> QuantConnect.Data.BaseData:
        """Gets the element in the collection at the current position of the enumerator."""
        ...

    def __init__(self, time_provider: QuantConnect.ITimeProvider, exchange_time_zone: typing.Any, trade_bar_aggregator: System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData], aux_data_enumerators: typing.List[System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData]]) -> None:
        """
        Initializes a new instance of the LiveAuxiliaryDataSynchronizingEnumerator class
        
        :param time_provider: The source of time used to gauge when this enumerator should emit extra bars when null data is returned from the source enumerator
        :param exchange_time_zone: The time zone the raw data is time stamped in
        :param trade_bar_aggregator: The trade bar aggregator enumerator
        :param aux_data_enumerators: The auxiliary data enumerators
        """
        ...

    def dispose(self) -> None:
        """Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources."""
        ...

    def move_next(self) -> bool:
        """
        Advances the enumerator to the next element of the collection.
        
        :returns: true if the enumerator was successfully advanced to the next element; false if the enumerator has passed the end of the collection.
        """
        ...

    def reset(self) -> None:
        """Sets the enumerator to its initial position, which is before the first element in the collection."""
        ...


class FrontierAwareEnumerator(System.Object, System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData]):
    """
    Provides an implementation of IEnumerator{BaseData} that will not emit
    data ahead of the frontier as specified by an instance of ITimeProvider.
    An instance of TimeZoneOffsetProvider is used to convert between UTC
    and the data's native time zone
    """

    @property
    def current(self) -> QuantConnect.Data.BaseData:
        """Gets the element in the collection at the current position of the enumerator."""
        ...

    def __init__(self, enumerator: System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData], time_provider: QuantConnect.ITimeProvider, offset_provider: QuantConnect.TimeZoneOffsetProvider) -> None:
        """
        Initializes a new instance of the FrontierAwareEnumerator class
        
        :param enumerator: The underlying enumerator to make frontier aware
        :param time_provider: The time provider used for resolving the current frontier time
        :param offset_provider: An offset provider used for converting the frontier UTC time into the data's native time zone
        """
        ...

    def dispose(self) -> None:
        """Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources."""
        ...

    def move_next(self) -> bool:
        """
        Advances the enumerator to the next element of the collection.
        
        :returns: true if the enumerator was successfully advanced to the next element; false if the enumerator has passed the end of the collection.
        """
        ...

    def reset(self) -> None:
        """Sets the enumerator to its initial position, which is before the first element in the collection."""
        ...


class LiveSubscriptionEnumerator(System.Object, System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData]):
    """Enumerator that will subscribe through the provided data queue handler and refresh the subscription if any mapping occurs"""

    @property
    def current(self) -> QuantConnect.Data.BaseData:
        """The current data object instance"""
        ...

    def __init__(self, data_config: QuantConnect.Data.SubscriptionDataConfig, data_queue_handler: QuantConnect.Interfaces.IDataQueueHandler, handler: typing.Callable[[System.Object, System.EventArgs], typing.Any], is_expired: typing.Callable[[QuantConnect.Data.SubscriptionDataConfig], bool]) -> None:
        """Creates a new instance"""
        ...

    def dispose(self) -> None:
        """Disposes of the used enumerators"""
        ...

    def move_next(self) -> bool:
        """Advances the enumerator to the next element."""
        ...

    def reset(self) -> None:
        """Reset the IEnumeration"""
        ...


class ScheduledEnumerator(System.Object, System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData]):
    """
    This enumerator will filter out data of the underlying enumerator based on a provided schedule.
    Will respect the schedule above the data, meaning will let older data through if the underlying provides none for the schedule date
    """

    @property
    def current(self) -> QuantConnect.Data.BaseData:
        """The current data point"""
        ...

    def __init__(self, underlying_enumerator: System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData], scheduled_times: typing.List[datetime.datetime], frontier_time_provider: QuantConnect.ITimeProvider, schedule_time_zone: typing.Any, start_time: typing.Union[datetime.datetime, datetime.date]) -> None:
        """
        Creates a new instance
        
        :param underlying_enumerator: The underlying enumerator to filter
        :param scheduled_times: The scheduled times to emit new data points
        :param frontier_time_provider: 
        :param schedule_time_zone: 
        :param start_time: the underlying request start time
        """
        ...

    def dispose(self) -> None:
        """Disposes of the underlying enumerator"""
        ...

    def move_next(self) -> bool:
        """
        Advances the enumerator to the next element of the collection.
        
        :returns: True if the enumerator was successfully advanced to the next element;
        false if the enumerator has passed the end of the collection.
        """
        ...

    def reset(self) -> None:
        """Resets the underlying enumerator"""
        ...


class PriceScaleFactorEnumerator(System.Object, System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData]):
    """
    This enumerator will update the SubscriptionDataConfig.price_scale_factor when required
    and adjust the raw BaseData prices based on the provided SubscriptionDataConfig.
    Assumes the prices of the provided IEnumerator are in raw mode.
    """

    @property
    def current(self) -> QuantConnect.Data.BaseData:
        """Last read BaseData object from this type and source"""
        ...

    def __init__(self, raw_data_enumerator: System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData], config: QuantConnect.Data.SubscriptionDataConfig, factor_file_provider: QuantConnect.Interfaces.IFactorFileProvider, live_mode: bool = False, end_date: typing.Optional[datetime.datetime] = None) -> None:
        """
        Creates a new instance of the PriceScaleFactorEnumerator.
        
        :param raw_data_enumerator: The underlying raw data enumerator
        :param config: The SubscriptionDataConfig to enumerate for.
        Will determine the DataNormalizationMode to use.
        :param factor_file_provider: The IFactorFileProvider instance to use
        :param live_mode: True, is this is a live mode data stream
        :param end_date: The enumerator end date
        """
        ...

    def dispose(self) -> None:
        """Dispose of the underlying enumerator."""
        ...

    def move_next(self) -> bool:
        """
        Advances the enumerator to the next element of the collection.
        
        :returns: True if the enumerator was successfully advanced to the next element;
        False if the enumerator has passed the end of the collection.
        """
        ...

    def reset(self) -> None:
        """Reset the IEnumeration"""
        ...


class RefreshEnumerator(typing.Generic[QuantConnect_Lean_Engine_DataFeeds_Enumerators_RefreshEnumerator_T], System.Object, System.Collections.Generic.IEnumerator[QuantConnect_Lean_Engine_DataFeeds_Enumerators_RefreshEnumerator_T]):
    """
    Provides an implementation of IEnumerator{T} that will
    always return true via MoveNext.
    """

    @property
    def current(self) -> QuantConnect_Lean_Engine_DataFeeds_Enumerators_RefreshEnumerator_T:
        """Gets the element in the collection at the current position of the enumerator."""
        ...

    def __init__(self, enumerator_factory: typing.Callable[[], System.Collections.Generic.IEnumerator[QuantConnect_Lean_Engine_DataFeeds_Enumerators_RefreshEnumerator_T]]) -> None:
        """
        Initializes a new instance of the RefreshEnumerator{T} class
        
        :param enumerator_factory: Enumerator factory used to regenerate the underlying
        enumerator when it ends
        """
        ...

    def dispose(self) -> None:
        """Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources."""
        ...

    def move_next(self) -> bool:
        """
        Advances the enumerator to the next element of the collection.
        
        :returns: true if the enumerator was successfully advanced to the next element; false if the enumerator has passed the end of the collection.
        """
        ...

    def reset(self) -> None:
        """Sets the enumerator to its initial position, which is before the first element in the collection."""
        ...


class SynchronizingEnumerator(typing.Generic[QuantConnect_Lean_Engine_DataFeeds_Enumerators_SynchronizingEnumerator_T], System.Object, System.Collections.Generic.IEnumerator[QuantConnect_Lean_Engine_DataFeeds_Enumerators_SynchronizingEnumerator_T], metaclass=abc.ABCMeta):
    """
    Represents an enumerator capable of synchronizing other enumerators of type T in time.
    This assumes that all enumerators have data time stamped in the same time zone
    """

    @property
    def current(self) -> QuantConnect_Lean_Engine_DataFeeds_Enumerators_SynchronizingEnumerator_T:
        """Gets the element in the collection at the current position of the enumerator."""
        ...

    @overload
    def __init__(self, *enumerators: typing.Union[System.Collections.Generic.IEnumerator[QuantConnect_Lean_Engine_DataFeeds_Enumerators_SynchronizingEnumerator_T], typing.Iterable[System.Collections.Generic.IEnumerator[QuantConnect_Lean_Engine_DataFeeds_Enumerators_SynchronizingEnumerator_T]]]) -> None:
        """
        Initializes a new instance of the SynchronizingEnumerator{T} class
        
        
        This codeEntityType is protected.
        
        :param enumerators: The enumerators to be synchronized. NOTE: Assumes the same time zone for all data
        """
        ...

    @overload
    def __init__(self, enumerators: typing.List[System.Collections.Generic.IEnumerator[QuantConnect_Lean_Engine_DataFeeds_Enumerators_SynchronizingEnumerator_T]]) -> None:
        """
        Initializes a new instance of the SynchronizingEnumerator{T} class
        
        
        This codeEntityType is protected.
        
        :param enumerators: The enumerators to be synchronized. NOTE: Assumes the same time zone for all data
        """
        ...

    def dispose(self) -> None:
        """Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources."""
        ...

    def get_instance_time(self, instance: QuantConnect_Lean_Engine_DataFeeds_Enumerators_SynchronizingEnumerator_T) -> datetime.datetime:
        """
        Gets the Timestamp for the data
        
        
        This codeEntityType is protected.
        """
        ...

    def move_next(self) -> bool:
        """
        Advances the enumerator to the next element of the collection.
        
        :returns: true if the enumerator was successfully advanced to the next element; false if the enumerator has passed the end of the collection.
        """
        ...

    def reset(self) -> None:
        """Sets the enumerator to its initial position, which is before the first element in the collection."""
        ...


class ITradableDateEventProvider(metaclass=abc.ABCMeta):
    """Interface for event providers for new tradable dates"""

    def get_events(self, event_args: QuantConnect.NewTradableDateEventArgs) -> typing.Iterable[QuantConnect.Data.BaseData]:
        """
        Called each time there is a new tradable day
        
        :param event_args: The new tradable day event arguments
        :returns: New corporate event if any.
        """
        ...

    def initialize(self, config: QuantConnect.Data.SubscriptionDataConfig, factor_file_provider: QuantConnect.Interfaces.IFactorFileProvider, map_file_provider: QuantConnect.Interfaces.IMapFileProvider, start_time: typing.Union[datetime.datetime, datetime.date]) -> None:
        """
        Initializes the event provider instance
        
        :param config: The SubscriptionDataConfig
        :param factor_file_provider: The factor file provider to use
        :param map_file_provider: The MapFile provider to use
        :param start_time: Start date for the data request
        """
        ...


class DividendEventProvider(System.Object, QuantConnect.Lean.Engine.DataFeeds.Enumerators.ITradableDateEventProvider):
    """Event provider who will emit Dividend events"""

    @property
    def factor_file(self) -> QuantConnect.Data.Auxiliary.CorporateFactorProvider:
        """
        The current instance being used
        
        
        This codeEntityType is protected.
        """
        ...

    @property
    def config(self) -> QuantConnect.Data.SubscriptionDataConfig:
        """
        The associated configuration
        
        
        This codeEntityType is protected.
        """
        ...

    def get_events(self, event_args: QuantConnect.NewTradableDateEventArgs) -> typing.Iterable[QuantConnect.Data.BaseData]:
        """
        Check for dividends and returns them
        
        :param event_args: The new tradable day event arguments
        :returns: New Dividend event if any.
        """
        ...

    def initialize(self, config: QuantConnect.Data.SubscriptionDataConfig, factor_file_provider: QuantConnect.Interfaces.IFactorFileProvider, map_file_provider: QuantConnect.Interfaces.IMapFileProvider, start_time: typing.Union[datetime.datetime, datetime.date]) -> None:
        """
        Initializes this instance
        
        :param config: The SubscriptionDataConfig
        :param factor_file_provider: The factor file provider to use
        :param map_file_provider: The Data.Auxiliary.MapFile provider to use
        :param start_time: Start date for the data request
        """
        ...

    def initialize_factor_file(self) -> None:
        """
        Initializes the factor file to use
        
        
        This codeEntityType is protected.
        """
        ...


class SubscriptionFilterEnumerator(System.Object, System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData]):
    """Implements a wrapper around a base data enumerator to provide a final filtering step"""

    @property
    def data_filter_error(self) -> _EventContainer[typing.Callable[[System.Object, System.Exception], typing.Any], typing.Any]:
        """Fired when there's an error executing a user's data filter"""
        ...

    @data_filter_error.setter
    def data_filter_error(self, value: _EventContainer[typing.Callable[[System.Object, System.Exception], typing.Any], typing.Any]) -> None:
        ...

    @property
    def current(self) -> QuantConnect.Data.BaseData:
        """Gets the element in the collection at the current position of the enumerator."""
        ...

    def __init__(self, enumerator: System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData], security: QuantConnect.Securities.Security, end_time: typing.Union[datetime.datetime, datetime.date], extended_market_hours: bool, live_mode: bool, security_exchange_hours: QuantConnect.Securities.SecurityExchangeHours) -> None:
        """
        Initializes a new instance of the SubscriptionFilterEnumerator class
        
        :param enumerator: The source enumerator to be wrapped
        :param security: The security containing an exchange and data filter
        :param end_time: The end time of the subscription
        :param extended_market_hours: True if extended market hours are enabled
        :param live_mode: True if live mode
        :param security_exchange_hours: The security exchange hours instance to use
        """
        ...

    def dispose(self) -> None:
        """Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources."""
        ...

    def move_next(self) -> bool:
        """
        Advances the enumerator to the next element of the collection.
        
        :returns: true if the enumerator was successfully advanced to the next element; false if the enumerator has passed the end of the collection.
        """
        ...

    def reset(self) -> None:
        """Sets the enumerator to its initial position, which is before the first element in the collection."""
        ...

    @staticmethod
    def wrap_for_data_feed(result_handler: QuantConnect.Lean.Engine.Results.IResultHandler, enumerator: System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData], security: QuantConnect.Securities.Security, end_time: typing.Union[datetime.datetime, datetime.date], extended_market_hours: bool, live_mode: bool, security_exchange_hours: QuantConnect.Securities.SecurityExchangeHours) -> QuantConnect.Lean.Engine.DataFeeds.Enumerators.SubscriptionFilterEnumerator:
        """
        Convenience method to wrap the enumerator and attach the data filter event to log and alery users of errors
        
        :param result_handler: Result handler reference used to send errors
        :param enumerator: The source enumerator to be wrapped
        :param security: The security who's data is being enumerated
        :param end_time: The end time of the subscription
        :param extended_market_hours: True if extended market hours are enabled
        :param live_mode: True if live mode
        :param security_exchange_hours: The security exchange hours instance to use
        :returns: A new instance of the SubscriptionFilterEnumerator class that has had it's data_filter_error
        event subscribed to to send errors to the result handler.
        """
        ...


class RateLimitEnumerator(typing.Generic[QuantConnect_Lean_Engine_DataFeeds_Enumerators_RateLimitEnumerator_T], System.Object, System.Collections.Generic.IEnumerator[QuantConnect_Lean_Engine_DataFeeds_Enumerators_RateLimitEnumerator_T]):
    """
    Provides augmentation of how often an enumerator can be called. Time is measured using
    an ITimeProvider instance and calls to the underlying enumerator are limited
    to a minimum time between each call.
    """

    @property
    def current(self) -> QuantConnect_Lean_Engine_DataFeeds_Enumerators_RateLimitEnumerator_T:
        """Gets the element in the collection at the current position of the enumerator."""
        ...

    def __init__(self, enumerator: System.Collections.Generic.IEnumerator[QuantConnect_Lean_Engine_DataFeeds_Enumerators_RateLimitEnumerator_T], time_provider: QuantConnect.ITimeProvider, minimum_time_between_calls: datetime.timedelta) -> None:
        """
        Initializes a new instance of the RateLimitEnumerator{T} class
        
        :param enumerator: The underlying enumerator to place rate limits on
        :param time_provider: Time provider used for determing the time between calls
        :param minimum_time_between_calls: The minimum time allowed between calls to the underlying enumerator
        """
        ...

    def dispose(self) -> None:
        """Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources."""
        ...

    def move_next(self) -> bool:
        """
        Advances the enumerator to the next element of the collection.
        
        :returns: true if the enumerator was successfully advanced to the next element; false if the enumerator has passed the end of the collection.
        """
        ...

    def reset(self) -> None:
        """Sets the enumerator to its initial position, which is before the first element in the collection."""
        ...


class FastForwardEnumerator(System.Object, System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData]):
    """Provides the ability to fast forward an enumerator based on the age of the data"""

    @property
    def current(self) -> QuantConnect.Data.BaseData:
        """Gets the element in the collection at the current position of the enumerator."""
        ...

    def __init__(self, enumerator: System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData], time_provider: QuantConnect.ITimeProvider, time_zone: typing.Any, maximum_data_age: datetime.timedelta) -> None:
        """
        Initializes a new instance of the FastForwardEnumerator class
        
        :param enumerator: The source enumerator
        :param time_provider: A time provider used to determine age of data
        :param time_zone: The data's time zone
        :param maximum_data_age: The maximum age of data allowed
        """
        ...

    def dispose(self) -> None:
        """Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources."""
        ...

    def move_next(self) -> bool:
        """
        Advances the enumerator to the next element of the collection.
        
        :returns: true if the enumerator was successfully advanced to the next element; false if the enumerator has passed the end of the collection.
        """
        ...

    def reset(self) -> None:
        """Sets the enumerator to its initial position, which is before the first element in the collection."""
        ...


class AuxiliaryDataEnumerator(System.Object, System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData]):
    """
    Auxiliary data enumerator that will, initialize and call the ITradableDateEventProvider.get_events
    implementation each time there is a new tradable day for every ITradableDateEventProvider
    provided.
    """

    @property
    def config(self) -> QuantConnect.Data.SubscriptionDataConfig:
        """
        The associated data configuration
        
        
        This codeEntityType is protected.
        """
        ...

    @property
    def current(self) -> QuantConnect.Data.BaseData:
        """Last read BaseData object from this type and source"""
        ...

    def __init__(self, config: QuantConnect.Data.SubscriptionDataConfig, factor_file_provider: QuantConnect.Interfaces.IFactorFileProvider, map_file_provider: QuantConnect.Interfaces.IMapFileProvider, tradable_date_event_providers: typing.List[QuantConnect.Lean.Engine.DataFeeds.Enumerators.ITradableDateEventProvider], tradable_day_notifier: QuantConnect.Lean.Engine.DataFeeds.Enumerators.ITradableDatesNotifier, start_time: typing.Union[datetime.datetime, datetime.date]) -> None:
        """
        Creates a new instance
        
        :param config: The SubscriptionDataConfig
        :param factor_file_provider: The factor file provider to use
        :param map_file_provider: The MapFile provider to use
        :param tradable_date_event_providers: The tradable dates event providers
        :param tradable_day_notifier: Tradable dates provider
        :param start_time: Start date for the data request
        """
        ...

    def dispose(self) -> None:
        """Dispose of the Stream Reader and close out the source stream and file connections."""
        ...

    def initialize(self) -> None:
        """
        Initializes the underlying tradable data event providers
        
        
        This codeEntityType is protected.
        """
        ...

    def move_next(self) -> bool:
        """
        Advances the enumerator to the next element.
        
        :returns: Always true.
        """
        ...

    def new_tradable_date(self, sender: typing.Any, event_args: QuantConnect.NewTradableDateEventArgs) -> None:
        """
        Handle a new tradable date, drives the ITradableDateEventProvider instances
        
        
        This codeEntityType is protected.
        """
        ...

    def reset(self) -> None:
        """Reset the IEnumeration"""
        ...


class MappingEventProvider(System.Object, QuantConnect.Lean.Engine.DataFeeds.Enumerators.ITradableDateEventProvider):
    """Event provider who will emit SymbolChangedEvent events"""

    @property
    def config(self) -> QuantConnect.Data.SubscriptionDataConfig:
        """
        The associated configuration
        
        
        This codeEntityType is protected.
        """
        ...

    @property
    def map_file(self) -> QuantConnect.Data.Auxiliary.MapFile:
        """
        The current instance being used
        
        
        This codeEntityType is protected.
        """
        ...

    def get_events(self, event_args: QuantConnect.NewTradableDateEventArgs) -> typing.Iterable[QuantConnect.Data.BaseData]:
        """
        Check for new mappings
        
        :param event_args: The new tradable day event arguments
        :returns: New mapping event if any.
        """
        ...

    def initialize(self, config: QuantConnect.Data.SubscriptionDataConfig, factor_file_provider: QuantConnect.Interfaces.IFactorFileProvider, map_file_provider: QuantConnect.Interfaces.IMapFileProvider, start_time: typing.Union[datetime.datetime, datetime.date]) -> None:
        """
        Initializes this instance
        
        :param config: The SubscriptionDataConfig
        :param factor_file_provider: The factor file provider to use
        :param map_file_provider: The Data.Auxiliary.MapFile provider to use
        :param start_time: Start date for the data request
        """
        ...

    def initialize_map_file(self) -> None:
        """
        Initializes the map file to use
        
        
        This codeEntityType is protected.
        """
        ...


class LiveMappingEventProvider(QuantConnect.Lean.Engine.DataFeeds.Enumerators.MappingEventProvider):
    """Event provider who will emit SymbolChangedEvent events"""

    def get_events(self, event_args: QuantConnect.NewTradableDateEventArgs) -> typing.Iterable[QuantConnect.Data.BaseData]:
        """Check for new mappings"""
        ...


class DelistingEventProvider(System.Object, QuantConnect.Lean.Engine.DataFeeds.Enumerators.ITradableDateEventProvider):
    """Event provider who will emit Delisting events"""

    @property
    def delisting_date(self) -> QuantConnect.Util.ReferenceWrapper[datetime.datetime]:
        """
        The delisting date
        
        
        This codeEntityType is protected.
        """
        ...

    @delisting_date.setter
    def delisting_date(self, value: QuantConnect.Util.ReferenceWrapper[datetime.datetime]) -> None:
        ...

    @property
    def map_file(self) -> QuantConnect.Data.Auxiliary.MapFile:
        """
        The current instance being used
        
        
        This codeEntityType is protected.
        """
        ...

    @property
    def config(self) -> QuantConnect.Data.SubscriptionDataConfig:
        """
        The associated configuration
        
        
        This codeEntityType is protected.
        """
        ...

    def get_events(self, event_args: QuantConnect.NewTradableDateEventArgs) -> typing.Iterable[QuantConnect.Data.BaseData]:
        """
        Check for delistings
        
        :param event_args: The new tradable day event arguments
        :returns: New delisting event if any.
        """
        ...

    def initialize(self, config: QuantConnect.Data.SubscriptionDataConfig, factor_file_provider: QuantConnect.Interfaces.IFactorFileProvider, map_file_provider: QuantConnect.Interfaces.IMapFileProvider, start_time: typing.Union[datetime.datetime, datetime.date]) -> None:
        """
        Initializes this instance
        
        :param config: The SubscriptionDataConfig
        :param factor_file_provider: The factor file provider to use
        :param map_file_provider: The Data.Auxiliary.MapFile provider to use
        :param start_time: Start date for the data request
        """
        ...

    def initialize_map_file(self) -> None:
        """
        Initializes the factor file to use
        
        
        This codeEntityType is protected.
        """
        ...


class LiveFillForwardEnumerator(QuantConnect.Lean.Engine.DataFeeds.Enumerators.FillForwardEnumerator):
    """
    An implementation of the FillForwardEnumerator that uses an ITimeProvider
    to determine if a fill forward bar needs to be emitted
    """

    def __init__(self, time_provider: QuantConnect.ITimeProvider, enumerator: System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData], exchange: QuantConnect.Securities.SecurityExchange, fill_forward_resolution: QuantConnect.Util.IReadOnlyRef[datetime.timedelta], is_extended_market_hours: bool, subscription_start_time: typing.Union[datetime.datetime, datetime.date], subscription_end_time: typing.Union[datetime.datetime, datetime.date], data_resolution: QuantConnect.Resolution, data_time_zone: typing.Any, daily_strict_end_time_enabled: bool, data_type: typing.Type = None, last_point_tracker: QuantConnect.Lean.Engine.DataFeeds.Enumerators.LastPointTracker = None) -> None:
        """
        Initializes a new instance of the LiveFillForwardEnumerator class that accepts
        a reference to the fill forward resolution, useful if the fill forward resolution is dynamic
        and changing as the enumeration progresses
        
        :param time_provider: The source of time used to gauage when this enumerator should emit extra bars when
        null data is returned from the source enumerator
        :param enumerator: The source enumerator to be filled forward
        :param exchange: The exchange used to determine when to insert fill forward data
        :param fill_forward_resolution: The resolution we'd like to receive data on
        :param is_extended_market_hours: True to use the exchange's extended market hours, false to use the regular market hours
        :param subscription_start_time: The start time of the subscription
        :param subscription_end_time: The end time of the subscription, once passing this date the enumerator will stop
        :param data_resolution: The source enumerator's data resolution
        :param data_time_zone: Time zone of the underlying source data
        :param daily_strict_end_time_enabled: True if daily strict end times are enabled
        :param data_type: The configuration data type this enumerator is for
        :param last_point_tracker: A reference to the last point emitted before this enumerator is first enumerated
        """
        ...

    @staticmethod
    def get_maximum_data_timeout(resolution: QuantConnect.Resolution) -> datetime.timedelta:
        """Helper method to know how much we should wait before fill forwarding a bar in live trading"""
        ...

    def requires_fill_forward_data(self, fill_forward_resolution: datetime.timedelta, previous: QuantConnect.Data.BaseData, next: QuantConnect.Data.BaseData, fill_forward: typing.Optional[QuantConnect.Data.BaseData]) -> typing.Tuple[bool, QuantConnect.Data.BaseData]:
        """
        Determines whether or not fill forward is required, and if true, will produce the new fill forward data
        
        
        This codeEntityType is protected.
        
        :param fill_forward_resolution: 
        :param previous: The last piece of data emitted by this enumerator
        :param next: The next piece of data on the source enumerator, this may be null
        :param fill_forward: When this function returns true, this will have a non-null value, null when the function returns false
        :returns: True when a new fill forward piece of data was produced and should be emitted by this enumerator.
        """
        ...


class LiveAuxiliaryDataEnumerator(QuantConnect.Lean.Engine.DataFeeds.Enumerators.AuxiliaryDataEnumerator):
    """Auxiliary data enumerator that will trigger new tradable dates event accordingly"""

    def __init__(self, config: QuantConnect.Data.SubscriptionDataConfig, factor_file_provider: QuantConnect.Interfaces.IFactorFileProvider, map_file_provider: QuantConnect.Interfaces.IMapFileProvider, tradable_date_event_providers: typing.List[QuantConnect.Lean.Engine.DataFeeds.Enumerators.ITradableDateEventProvider], start_time: typing.Union[datetime.datetime, datetime.date], time_provider: QuantConnect.ITimeProvider, security_cache: QuantConnect.Securities.SecurityCache) -> None:
        """
        Creates a new instance
        
        :param config: The SubscriptionDataConfig
        :param factor_file_provider: The factor file provider to use
        :param map_file_provider: The MapFile provider to use
        :param tradable_date_event_providers: The tradable dates event providers
        :param start_time: Start date for the data request
        :param time_provider: The time provider to use
        :param security_cache: The security cache
        """
        ...

    def move_next(self) -> bool:
        """Moves the LiveAuxiliaryDataEnumerator to the next item"""
        ...

    @staticmethod
    def try_create(data_config: QuantConnect.Data.SubscriptionDataConfig, time_provider: QuantConnect.ITimeProvider, security_cache: QuantConnect.Securities.SecurityCache, map_file_provider: QuantConnect.Interfaces.IMapFileProvider, file_provider: QuantConnect.Interfaces.IFactorFileProvider, start_time: typing.Union[datetime.datetime, datetime.date], enumerator: typing.Optional[System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData]]) -> typing.Tuple[bool, System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData]]:
        """
        Helper method to create a new instance.
        Knows which security types should create one and determines the appropriate delisting event provider to use
        """
        ...


class SynchronizingSliceEnumerator(QuantConnect.Lean.Engine.DataFeeds.Enumerators.SynchronizingEnumerator[QuantConnect.Data.Slice]):
    """
    Represents an enumerator capable of synchronizing other slice enumerators in time.
    This assumes that all enumerators have data time stamped in the same time zone
    """

    @overload
    def __init__(self, *enumerators: typing.Union[System.Collections.Generic.IEnumerator[QuantConnect.Data.Slice], typing.Iterable[System.Collections.Generic.IEnumerator[QuantConnect.Data.Slice]]]) -> None:
        """
        Initializes a new instance of the SynchronizingSliceEnumerator class
        
        :param enumerators: The enumerators to be synchronized. NOTE: Assumes the same time zone for all data
        """
        ...

    @overload
    def __init__(self, enumerators: typing.List[System.Collections.IEnumerator]) -> None:
        """
        Initializes a new instance of the SynchronizingSliceEnumerator class
        
        :param enumerators: The enumerators to be synchronized. NOTE: Assumes the same time zone for all data
        """
        ...

    def get_instance_time(self, instance: QuantConnect.Data.Slice) -> datetime.datetime:
        """
        Gets the Timestamp for the data
        
        
        This codeEntityType is protected.
        """
        ...


class LiveDividendEventProvider(QuantConnect.Lean.Engine.DataFeeds.Enumerators.DividendEventProvider):
    """Event provider who will emit SymbolChangedEvent events"""

    def get_events(self, event_args: QuantConnect.NewTradableDateEventArgs) -> typing.Iterable[QuantConnect.Data.BaseData]:
        """
        Check for dividends and returns them
        
        :param event_args: The new tradable day event arguments
        :returns: New Dividend event if any.
        """
        ...


class NewDataAvailableEventArgs(System.EventArgs):
    """Event args for when a new data point is ready to be emitted"""

    @property
    def data_point(self) -> QuantConnect.Data.IBaseData:
        """The new data point"""
        ...

    @data_point.setter
    def data_point(self, value: QuantConnect.Data.IBaseData) -> None:
        ...


class LiveDelistingEventProvider(QuantConnect.Lean.Engine.DataFeeds.Enumerators.DelistingEventProvider):
    """Delisting event provider implementation which will source the delisting date based on new map files"""

    def get_events(self, event_args: QuantConnect.NewTradableDateEventArgs) -> typing.Iterable[QuantConnect.Data.BaseData]:
        """
        Check for delistings
        
        :param event_args: The new tradable day event arguments
        :returns: New delisting event if any.
        """
        ...


class ScannableEnumerator(typing.Generic[QuantConnect_Lean_Engine_DataFeeds_Enumerators_ScannableEnumerator_T], System.Object, System.Collections.Generic.IEnumerator[QuantConnect_Lean_Engine_DataFeeds_Enumerators_ScannableEnumerator_T]):
    """An implementation of IEnumerator{T} that relies on "consolidated" data"""

    @property
    def current(self) -> QuantConnect_Lean_Engine_DataFeeds_Enumerators_ScannableEnumerator_T:
        """Gets the element in the collection at the current position of the enumerator."""
        ...

    def __init__(self, consolidator: typing.Union[QuantConnect.Data.Consolidators.IDataConsolidator, QuantConnect.Python.PythonConsolidator, datetime.timedelta], time_zone: typing.Any, time_provider: QuantConnect.ITimeProvider, new_data_available_handler: typing.Callable[[System.Object, System.EventArgs], typing.Any], is_period_based: bool = True) -> None:
        """
        Initializes a new instance of the ScannableEnumerator{T} class
        
        :param consolidator: Consolidator taking BaseData updates and firing events containing new 'consolidated' data
        :param time_zone: The time zone the raw data is time stamped in
        :param time_provider: The time provider instance used to determine when bars are completed and can be emitted
        :param new_data_available_handler: The event handler for a new available data point
        :param is_period_based: The consolidator is period based, this will enable scanning on move_next
        """
        ...

    def dispose(self) -> None:
        """Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources."""
        ...

    def move_next(self) -> bool:
        """
        Advances the enumerator to the next element of the collection.
        
        :returns: true if the enumerator was successfully advanced to the next element; false if the enumerator has passed the end of the collection.
        """
        ...

    def reset(self) -> None:
        """Sets the enumerator to its initial position, which is before the first element in the collection."""
        ...

    def update(self, data: QuantConnect_Lean_Engine_DataFeeds_Enumerators_ScannableEnumerator_T) -> None:
        """
        Updates the consolidator
        
        :param data: The data to consolidate
        """
        ...


class SubscriptionDataEnumerator(System.Object, System.Collections.Generic.IEnumerator[QuantConnect.Lean.Engine.DataFeeds.SubscriptionData]):
    """An IEnumerator{SubscriptionData} which wraps an existing IEnumerator{BaseData}."""

    @property
    def current(self) -> QuantConnect.Lean.Engine.DataFeeds.SubscriptionData:
        """Gets the element in the collection at the current position of the enumerator."""
        ...

    def __init__(self, configuration: QuantConnect.Data.SubscriptionDataConfig, exchange_hours: QuantConnect.Securities.SecurityExchangeHours, offset_provider: QuantConnect.TimeZoneOffsetProvider, enumerator: System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData], is_universe: bool, daily_strict_end_time_enabled: bool) -> None:
        """
        Creates a new instance
        
        :param configuration: The subscription's configuration
        :param exchange_hours: The security's exchange hours
        :param offset_provider: The subscription's time zone offset provider
        :param enumerator: The underlying data enumerator
        :param is_universe: The subscription is a universe subscription
        :returns: A subscription data enumerator.
        """
        ...

    def dispose(self) -> None:
        """Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources."""
        ...

    def move_next(self) -> bool:
        """
        Advances the enumerator to the next element of the collection.
        
        :returns: True if the enumerator was successfully advanced to the next element;
        False if the enumerator has passed the end of the collection.
        """
        ...

    def reset(self) -> None:
        """Sets the enumerator to its initial position, which is before the first element in the collection."""
        ...


class BaseDataCollectionAggregatorEnumerator(System.Object, System.Collections.Generic.IEnumerator[QuantConnect.Data.UniverseSelection.BaseDataCollection]):
    """
    Provides an implementation of IEnumerator{BaseDataCollection}
    that aggregates an underlying IEnumerator{BaseData} into a single
    data packet
    """

    @property
    def current(self) -> QuantConnect.Data.UniverseSelection.BaseDataCollection:
        """Gets the element in the collection at the current position of the enumerator."""
        ...

    def __init__(self, enumerator: System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData], symbol: typing.Union[QuantConnect.Symbol, str, QuantConnect.Data.Market.BaseContract], live_mode: bool = False) -> None:
        """
        Initializes a new instance of the BaseDataCollectionAggregatorEnumerator class
        This will aggregate instances emitted from the underlying enumerator and tag them with the
        specified symbol
        
        :param enumerator: The underlying enumerator to aggregate
        :param symbol: The symbol to place on the aggregated collection
        :param live_mode: True if running in live mode
        """
        ...

    def dispose(self) -> None:
        """Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources."""
        ...

    def move_next(self) -> bool:
        """
        Advances the enumerator to the next element of the collection.
        
        :returns: true if the enumerator was successfully advanced to the next element; false if the enumerator has passed the end of the collection.
        """
        ...

    def reset(self) -> None:
        """Sets the enumerator to its initial position, which is before the first element in the collection."""
        ...


class SplitEventProvider(System.Object, QuantConnect.Lean.Engine.DataFeeds.Enumerators.ITradableDateEventProvider):
    """Event provider who will emit Split events"""

    @property
    def factor_file(self) -> QuantConnect.Data.Auxiliary.CorporateFactorProvider:
        """
        The current instance being used
        
        
        This codeEntityType is protected.
        """
        ...

    @property
    def config(self) -> QuantConnect.Data.SubscriptionDataConfig:
        """
        The associated configuration
        
        
        This codeEntityType is protected.
        """
        ...

    def get_events(self, event_args: QuantConnect.NewTradableDateEventArgs) -> typing.Iterable[QuantConnect.Data.BaseData]:
        """
        Check for new splits
        
        :param event_args: The new tradable day event arguments
        :returns: New split event if any.
        """
        ...

    def initialize(self, config: QuantConnect.Data.SubscriptionDataConfig, factor_file_provider: QuantConnect.Interfaces.IFactorFileProvider, map_file_provider: QuantConnect.Interfaces.IMapFileProvider, start_time: typing.Union[datetime.datetime, datetime.date]) -> None:
        """
        Initializes this instance
        
        :param config: The SubscriptionDataConfig
        :param factor_file_provider: The factor file provider to use
        :param map_file_provider: The Data.Auxiliary.MapFile provider to use
        :param start_time: Start date for the data request
        """
        ...

    def initialize_factor_file(self) -> None:
        """
        Initializes the factor file to use
        
        
        This codeEntityType is protected.
        """
        ...


class LiveSplitEventProvider(QuantConnect.Lean.Engine.DataFeeds.Enumerators.SplitEventProvider):
    """Event provider who will emit SymbolChangedEvent events"""

    def get_events(self, event_args: QuantConnect.NewTradableDateEventArgs) -> typing.Iterable[QuantConnect.Data.BaseData]:
        """
        Check for dividends and returns them
        
        :param event_args: The new tradable day event arguments
        :returns: New Dividend event if any.
        """
        ...


class _EventContainer(typing.Generic[QuantConnect_Lean_Engine_DataFeeds_Enumerators__EventContainer_Callable, QuantConnect_Lean_Engine_DataFeeds_Enumerators__EventContainer_ReturnType]):
    """This class is used to provide accurate autocomplete on events and cannot be imported."""

    def __call__(self, *args: typing.Any, **kwargs: typing.Any) -> QuantConnect_Lean_Engine_DataFeeds_Enumerators__EventContainer_ReturnType:
        """Fires the event."""
        ...

    def __iadd__(self, item: QuantConnect_Lean_Engine_DataFeeds_Enumerators__EventContainer_Callable) -> typing.Self:
        """Registers an event handler."""
        ...

    def __isub__(self, item: QuantConnect_Lean_Engine_DataFeeds_Enumerators__EventContainer_Callable) -> typing.Self:
        """Unregisters an event handler."""
        ...





from typing import overload
from enum import IntEnum
import abc
import datetime
import typing

import QuantConnect
import QuantConnect.Algorithm.Framework.Alphas
import QuantConnect.Notifications
import QuantConnect.Orders
import QuantConnect.Packets
import QuantConnect.Securities
import QuantConnect.Statistics
import System
import System.Collections.Generic


class BaseResultParameters(System.Object):
    """Base parameters used by LiveResultParameters and BacktestResultParameters"""

    @property
    def profit_loss(self) -> System.Collections.Generic.IDictionary[datetime.datetime, float]:
        """Trade profit and loss information since the last algorithm result packet"""
        ...

    @profit_loss.setter
    def profit_loss(self, value: System.Collections.Generic.IDictionary[datetime.datetime, float]) -> None:
        ...

    @property
    def charts(self) -> System.Collections.Generic.IDictionary[str, QuantConnect.Chart]:
        """Charts updates for the live algorithm since the last result packet"""
        ...

    @charts.setter
    def charts(self, value: System.Collections.Generic.IDictionary[str, QuantConnect.Chart]) -> None:
        ...

    @property
    def orders(self) -> System.Collections.Generic.IDictionary[int, QuantConnect.Orders.Order]:
        """Order updates since the last result packet"""
        ...

    @orders.setter
    def orders(self, value: System.Collections.Generic.IDictionary[int, QuantConnect.Orders.Order]) -> None:
        ...

    @property
    def order_events(self) -> typing.List[QuantConnect.Orders.OrderEvent]:
        """Order events updates since the last result packet"""
        ...

    @order_events.setter
    def order_events(self, value: typing.List[QuantConnect.Orders.OrderEvent]) -> None:
        ...

    @property
    def statistics(self) -> System.Collections.Generic.IDictionary[str, str]:
        """Statistics information sent during the algorithm operations."""
        ...

    @statistics.setter
    def statistics(self, value: System.Collections.Generic.IDictionary[str, str]) -> None:
        ...

    @property
    def runtime_statistics(self) -> System.Collections.Generic.IDictionary[str, str]:
        """Runtime banner/updating statistics in the title banner of the live algorithm GUI."""
        ...

    @runtime_statistics.setter
    def runtime_statistics(self, value: System.Collections.Generic.IDictionary[str, str]) -> None:
        ...

    @property
    def state(self) -> System.Collections.Generic.IDictionary[str, str]:
        """State information of the algorithm."""
        ...

    @state.setter
    def state(self, value: System.Collections.Generic.IDictionary[str, str]) -> None:
        ...

    @property
    def algorithm_configuration(self) -> QuantConnect.AlgorithmConfiguration:
        """The algorithm's configuration required for report generation"""
        ...

    @algorithm_configuration.setter
    def algorithm_configuration(self, value: QuantConnect.AlgorithmConfiguration) -> None:
        ...

    def __init__(self, charts: System.Collections.Generic.IDictionary[str, QuantConnect.Chart], orders: System.Collections.Generic.IDictionary[int, QuantConnect.Orders.Order], profit_loss: System.Collections.Generic.IDictionary[datetime.datetime, float], statistics: System.Collections.Generic.IDictionary[str, str], runtime_statistics: System.Collections.Generic.IDictionary[str, str], order_events: typing.List[QuantConnect.Orders.OrderEvent], algorithm_configuration: QuantConnect.AlgorithmConfiguration = None, state: System.Collections.Generic.IDictionary[str, str] = None) -> None:
        """Creates a new instance"""
        ...


class BacktestResultParameters(QuantConnect.Packets.BaseResultParameters):
    """Defines the parameters for BacktestResult"""

    @property
    def rolling_window(self) -> System.Collections.Generic.Dictionary[str, QuantConnect.Statistics.AlgorithmPerformance]:
        """Rolling window detailed statistics."""
        ...

    @rolling_window.setter
    def rolling_window(self, value: System.Collections.Generic.Dictionary[str, QuantConnect.Statistics.AlgorithmPerformance]) -> None:
        ...

    @property
    def total_performance(self) -> QuantConnect.Statistics.AlgorithmPerformance:
        """Rolling window detailed statistics."""
        ...

    @total_performance.setter
    def total_performance(self, value: QuantConnect.Statistics.AlgorithmPerformance) -> None:
        ...

    def __init__(self, charts: System.Collections.Generic.IDictionary[str, QuantConnect.Chart], orders: System.Collections.Generic.IDictionary[int, QuantConnect.Orders.Order], profit_loss: System.Collections.Generic.IDictionary[datetime.datetime, float], statistics: System.Collections.Generic.IDictionary[str, str], runtime_statistics: System.Collections.Generic.IDictionary[str, str], rolling_window: System.Collections.Generic.Dictionary[str, QuantConnect.Statistics.AlgorithmPerformance], order_events: typing.List[QuantConnect.Orders.OrderEvent], total_performance: QuantConnect.Statistics.AlgorithmPerformance = None, algorithm_configuration: QuantConnect.AlgorithmConfiguration = None, state: System.Collections.Generic.IDictionary[str, str] = None) -> None:
        """Creates a new instance"""
        ...


class PacketType(IntEnum):
    """Classifications of internal packet system"""

    NONE = 0

    ALGORITHM_NODE = 1

    AUTOCOMPLETE_WORK = 2

    AUTOCOMPLETE_RESULT = 3

    BACKTEST_NODE = 4

    BACKTEST_RESULT = 5

    BACKTEST_WORK = 6

    LIVE_NODE = 7

    LIVE_RESULT = 8

    LIVE_WORK = 9

    SECURITY_TYPES = 10

    BACKTEST_ERROR = 11

    ALGORITHM_STATUS = 12

    BUILD_WORK = 13

    BUILD_SUCCESS = 14

    BUILD_ERROR = 15

    RUNTIME_ERROR = 16

    HANDLED_ERROR = 17

    LOG = 18

    DEBUG = 19

    ORDER_EVENT = 20

    SUCCESS = 21

    HISTORY = 22

    COMMAND_RESULT = 23

    GIT_HUB_HOOK = 24

    DOCUMENTATION_RESULT = 25

    DOCUMENTATION = 26

    SYSTEM_DEBUG = 27

    ALPHA_RESULT = 28

    ALPHA_WORK = 29

    ALPHA_NODE = 30

    REGRESSION_ALGORITHM = 31

    ALPHA_HEARTBEAT = 32

    DEBUGGING_STATUS = 33

    OPTIMIZATION_NODE = 34

    OPTIMIZATION_ESTIMATE = 35

    OPTIMIZATION_STATUS = 36

    OPTIMIZATION_RESULT = 37

    AGGREGATED = 38

    LANGUAGE_MODEL_QUERY = 39

    LANGUAGE_MODEL_FEEDBACK = 40

    LANGUAGE_MODEL_RESPONSE = 41

    LANGUAGE_MODEL_CODE_ANALYSIS = 42

    LANGUAGE_MODEL_CHAT_WORK = 43

    LANGUAGE_MODEL_CHAT_RESPONSE = 44

    ALGORITHM_NAME_UPDATE = 45

    ALGORITHM_TAGS_UPDATE = 46

    RESEARCH_NODE = 47

    ORGANIZATION_UPDATE = 48

    BUILD_WARNING = 49

    LANGUAGE_MODEL_FUNCTION_CALL = 50

    LANGUAGE_MODEL_AGENT_MESSAGE = 51


class Packet(System.Object):
    """Base class for packet messaging system"""

    @property
    def type(self) -> QuantConnect.Packets.PacketType:
        """Packet type defined by a string enum"""
        ...

    @type.setter
    def type(self, value: QuantConnect.Packets.PacketType) -> None:
        ...

    @property
    def channel(self) -> str:
        """User unique specific channel endpoint to send the packets"""
        ...

    @channel.setter
    def channel(self, value: str) -> None:
        ...

    def __init__(self, type: QuantConnect.Packets.PacketType) -> None:
        """
        Initialize the base class and setup the packet type.
        
        :param type: PacketType for the class.
        """
        ...


class PythonEnvironmentPacket(QuantConnect.Packets.Packet, metaclass=abc.ABCMeta):
    """
    Python Environment Packet is an abstract packet that contains a PythonVirtualEnvironment
    definition. Intended to be used by inheriting classes that may use a PythonVirtualEnvironment
    """

    @property
    def python_virtual_environment(self) -> str:
        """
        Virtual environment ID used to find PythonEvironments
        Ideally MD5, but environment names work as well.
        """
        ...

    @python_virtual_environment.setter
    def python_virtual_environment(self, value: str) -> None:
        ...

    def __init__(self, type: QuantConnect.Packets.PacketType) -> None:
        """
        Default constructor for a PythonEnvironmentPacket
        
        
        This codeEntityType is protected.
        
        :param type: 
        """
        ...


class LeakyBucketControlParameters(System.Object):
    """
    Provides parameters that control the behavior of a leaky bucket rate limiting algorithm. The
    parameter names below are phrased in the positive, such that the bucket is filled up over time
    vs leaking out over time.
    """

    default_capacity: int

    default_time_interval: int
    """Default time interval"""

    default_refill_amount: int
    """Default refill amount"""

    @property
    def capacity(self) -> int:
        """
        Sets the total capacity of the bucket in a leaky bucket algorithm. This is the maximum
        number of 'units' the bucket can hold and also defines the maximum burst rate, assuming
        instantaneous usage of 'units'. In reality, the usage of 'units' takes times, and so it
        is possible for the bucket to incrementally refill while consuming from the bucket.
        """
        ...

    @capacity.setter
    def capacity(self, value: int) -> None:
        ...

    @property
    def refill_amount(self) -> int:
        """
        Sets the refill amount of the bucket. This defines the quantity of 'units' that become available
        to a consuming entity after the time interval has elapsed. For example, if the refill amount is
        equal to one, then each time interval one new 'unit' will be made available for a consumer that is
        throttled by the leaky bucket.
        """
        ...

    @refill_amount.setter
    def refill_amount(self, value: int) -> None:
        ...

    @property
    def time_interval_minutes(self) -> int:
        """
        Sets the time interval for the refill amount of the bucket, in minutes. After this amount of wall-clock
        time has passed, the bucket will refill the refill amount, thereby making more 'units' available
        for a consumer. For example, if the refill amount equals 10 and the time interval is 30 minutes, then
        every 30 minutes, 10 more 'units' become available for a consumer. The available 'units' will
        continue to increase until the bucket capacity is reached.
        """
        ...

    @time_interval_minutes.setter
    def time_interval_minutes(self, value: int) -> None:
        ...

    @overload
    def __init__(self) -> None:
        """Initializes a new instance of the LeakyBucketControlParameters using default values"""
        ...

    @overload
    def __init__(self, capacity: int, refill_amount: int, time_interval_minutes: int) -> None:
        """
        Initializes a new instance of the LeakyBucketControlParameters with the specified value
        
        :param capacity: The total capacity of the bucket in minutes
        :param refill_amount: The number of additional minutes to add to the bucket
        after time_interval_minutes has elapsed
        :param time_interval_minutes: The interval, in minutes, that must pass before the refill_amount
        is added back to the bucket for reuse
        """
        ...


class StoragePermissions(System.Object):
    """Holds the permissions for the object store"""

    @property
    def read(self) -> bool:
        """Whether the user has read permissions on the object store"""
        ...

    @read.setter
    def read(self, value: bool) -> None:
        ...

    @property
    def write(self) -> bool:
        """Whether the user has write permissions on the object store"""
        ...

    @write.setter
    def write(self, value: bool) -> None:
        ...

    @property
    def delete(self) -> bool:
        """Whether the user has delete permissions on the object store"""
        ...

    @delete.setter
    def delete(self, value: bool) -> None:
        ...

    def __init__(self) -> None:
        """Initializes a new instance of the StoragePermissions struct with default permissions."""
        ...

    def to_string(self) -> str:
        """Returns a string representation of the storage permissions."""
        ...


class Controls(System.Object):
    """Specifies values used to control algorithm limits"""

    @property
    def maximum_runtime_minutes(self) -> int:
        """The maximum runtime in minutes"""
        ...

    @maximum_runtime_minutes.setter
    def maximum_runtime_minutes(self, value: int) -> None:
        ...

    @property
    def minute_limit(self) -> int:
        """The maximum number of minute symbols"""
        ...

    @minute_limit.setter
    def minute_limit(self, value: int) -> None:
        ...

    @property
    def second_limit(self) -> int:
        """The maximum number of second symbols"""
        ...

    @second_limit.setter
    def second_limit(self, value: int) -> None:
        ...

    @property
    def tick_limit(self) -> int:
        """The maximum number of tick symbol"""
        ...

    @tick_limit.setter
    def tick_limit(self, value: int) -> None:
        ...

    @property
    def ram_allocation(self) -> int:
        """Ram allocation for this algorithm in MB"""
        ...

    @ram_allocation.setter
    def ram_allocation(self, value: int) -> None:
        ...

    @property
    def cpu_allocation(self) -> float:
        """CPU allocation for this algorithm"""
        ...

    @cpu_allocation.setter
    def cpu_allocation(self, value: float) -> None:
        ...

    @property
    def live_log_limit(self) -> int:
        """The user live log limit"""
        ...

    @live_log_limit.setter
    def live_log_limit(self, value: int) -> None:
        ...

    @property
    def backtest_log_limit(self) -> int:
        """The user backtesting log limit"""
        ...

    @backtest_log_limit.setter
    def backtest_log_limit(self, value: int) -> None:
        ...

    @property
    def daily_log_limit(self) -> int:
        """The daily log limit of a user"""
        ...

    @daily_log_limit.setter
    def daily_log_limit(self, value: int) -> None:
        ...

    @property
    def remaining_log_allowance(self) -> int:
        """The remaining log allowance for a user"""
        ...

    @remaining_log_allowance.setter
    def remaining_log_allowance(self, value: int) -> None:
        ...

    @property
    def backtesting_max_insights(self) -> int:
        """Maximimum number of insights we'll store and score in a single backtest"""
        ...

    @backtesting_max_insights.setter
    def backtesting_max_insights(self, value: int) -> None:
        ...

    @property
    def backtesting_max_orders(self) -> int:
        """Maximimum number of orders we'll allow in a backtest."""
        ...

    @backtesting_max_orders.setter
    def backtesting_max_orders(self, value: int) -> None:
        ...

    @property
    def maximum_data_points_per_chart_series(self) -> int:
        """Limits the amount of data points per chart series. Applies only for backtesting"""
        ...

    @maximum_data_points_per_chart_series.setter
    def maximum_data_points_per_chart_series(self, value: int) -> None:
        ...

    @property
    def maximum_chart_series(self) -> int:
        """Limits the amount of chart series. Applies only for backtesting"""
        ...

    @maximum_chart_series.setter
    def maximum_chart_series(self, value: int) -> None:
        ...

    @property
    def second_time_out(self) -> int:
        """The amount seconds used for timeout limits"""
        ...

    @second_time_out.setter
    def second_time_out(self, value: int) -> None:
        ...

    @property
    def training_limits(self) -> QuantConnect.Packets.LeakyBucketControlParameters:
        """
        Sets parameters used for determining the behavior of the leaky bucket algorithm that
        controls how much time is available for an algorithm to use the training feature.
        """
        ...

    @training_limits.setter
    def training_limits(self, value: QuantConnect.Packets.LeakyBucketControlParameters) -> None:
        ...

    @property
    def storage_limit(self) -> int:
        """Limits the total size of storage used by IObjectStore"""
        ...

    @storage_limit.setter
    def storage_limit(self, value: int) -> None:
        ...

    @property
    def storage_file_count(self) -> int:
        """Limits the number of files to be held under the IObjectStore"""
        ...

    @storage_file_count.setter
    def storage_file_count(self, value: int) -> None:
        ...

    @property
    def storage_access(self) -> QuantConnect.Packets.StoragePermissions:
        """Holds the permissions for the object store"""
        ...

    @storage_access.setter
    def storage_access(self, value: QuantConnect.Packets.StoragePermissions) -> None:
        ...

    @property
    def persistence_interval_seconds(self) -> int:
        """
        The interval over which the IObjectStore will persistence the contents of
        the object store
        """
        ...

    @persistence_interval_seconds.setter
    def persistence_interval_seconds(self, value: int) -> None:
        ...

    @property
    def credit_cost(self) -> float:
        """The cost associated with running this job"""
        ...

    @credit_cost.setter
    def credit_cost(self, value: float) -> None:
        ...

    def __init__(self) -> None:
        """Initializes a new default instance of the Controls class"""
        ...


class AlgorithmNodePacket(QuantConnect.Packets.PythonEnvironmentPacket):
    """Algorithm Node Packet is a work task for the Lean Engine"""

    @property
    def host_name(self) -> str:
        """The host name to use if any"""
        ...

    @host_name.setter
    def host_name(self, value: str) -> None:
        ...

    @property
    def user_id(self) -> int:
        """User Id placing request"""
        ...

    @user_id.setter
    def user_id(self, value: int) -> None:
        ...

    @property
    def user_token(self) -> str:
        ...

    @user_token.setter
    def user_token(self, value: str) -> None:
        ...

    @property
    def organization_id(self) -> str:
        ...

    @organization_id.setter
    def organization_id(self, value: str) -> None:
        ...

    @property
    def project_id(self) -> int:
        """Project Id of the request"""
        ...

    @project_id.setter
    def project_id(self, value: int) -> None:
        ...

    @property
    def project_name(self) -> str:
        """Project name of the request"""
        ...

    @project_name.setter
    def project_name(self, value: str) -> None:
        ...

    @property
    def algorithm_id(self) -> str:
        """Algorithm Id - BacktestId or DeployId - Common Id property between packets."""
        ...

    @property
    def session_id(self) -> str:
        """User session Id for authentication"""
        ...

    @session_id.setter
    def session_id(self, value: str) -> None:
        ...

    @property
    def language(self) -> QuantConnect.Language:
        """Language flag: Currently represents IL code or Dynamic Scripted Types."""
        ...

    @language.setter
    def language(self, value: QuantConnect.Language) -> None:
        ...

    @property
    def server_type(self) -> QuantConnect.ServerType:
        """Server type for the deployment (512, 1024, 2048)"""
        ...

    @server_type.setter
    def server_type(self, value: QuantConnect.ServerType) -> None:
        ...

    @property
    def compile_id(self) -> str:
        """Unique compile id of this backtest"""
        ...

    @compile_id.setter
    def compile_id(self, value: str) -> None:
        ...

    @property
    def version(self) -> str:
        """Version number identifier for the lean engine."""
        ...

    @version.setter
    def version(self, value: str) -> None:
        ...

    @property
    def redelivered(self) -> bool:
        """
        An algorithm packet which has already been run and is being redelivered on this node.
        In this event we don't want to relaunch the task as it may result in unexpected behaviour for user.
        """
        ...

    @redelivered.setter
    def redelivered(self, value: bool) -> None:
        ...

    @property
    def algorithm(self) -> typing.List[int]:
        """Algorithm binary with zip of contents"""
        ...

    @algorithm.setter
    def algorithm(self, value: typing.List[int]) -> None:
        ...

    @property
    def request_source(self) -> str:
        """Request source - Web IDE or API - for controling result handler behaviour"""
        ...

    @request_source.setter
    def request_source(self, value: str) -> None:
        ...

    @property
    def ram_allocation(self) -> int:
        """The maximum amount of RAM (in MB) this algorithm is allowed to utilize"""
        ...

    @property
    def controls(self) -> QuantConnect.Packets.Controls:
        """Specifies values to control algorithm limits"""
        ...

    @controls.setter
    def controls(self, value: QuantConnect.Packets.Controls) -> None:
        ...

    @property
    def parameters(self) -> System.Collections.Generic.Dictionary[str, str]:
        """The parameter values used to set algorithm parameters"""
        ...

    @parameters.setter
    def parameters(self, value: System.Collections.Generic.Dictionary[str, str]) -> None:
        ...

    @property
    def history_provider(self) -> str:
        """String name of the HistoryProvider we're running with"""
        ...

    @history_provider.setter
    def history_provider(self, value: str) -> None:
        ...

    @property
    def algorithm_mode(self) -> QuantConnect.AlgorithmMode:
        """Algorithm running mode."""
        ...

    @property
    def deployment_target(self) -> QuantConnect.DeploymentTarget:
        """Deployment target, either local or cloud."""
        ...

    @deployment_target.setter
    def deployment_target(self, value: QuantConnect.DeploymentTarget) -> None:
        ...

    def __init__(self, type: QuantConnect.Packets.PacketType) -> None:
        """
        Default constructor for the algorithm node:
        
        :param type: 
        """
        ...

    def get_algorithm_name(self) -> str:
        """Gets a unique name for the algorithm defined by this packet"""
        ...


class BacktestNodePacket(QuantConnect.Packets.AlgorithmNodePacket):
    """Algorithm backtest task information packet."""

    @property
    def name(self) -> str:
        """Name of the backtest as randomly defined in the IDE."""
        ...

    @name.setter
    def name(self, value: str) -> None:
        ...

    @property
    def backtest_id(self) -> str:
        """BacktestId / Algorithm Id for this task"""
        ...

    @backtest_id.setter
    def backtest_id(self, value: str) -> None:
        ...

    @property
    def optimization_id(self) -> str:
        """Optimization Id for this task"""
        ...

    @optimization_id.setter
    def optimization_id(self, value: str) -> None:
        ...

    @property
    def period_start(self) -> typing.Optional[datetime.datetime]:
        """Backtest start-date as defined in the Initialize() method."""
        ...

    @period_start.setter
    def period_start(self, value: typing.Optional[datetime.datetime]) -> None:
        ...

    @property
    def period_finish(self) -> typing.Optional[datetime.datetime]:
        """Backtest end date as defined in the Initialize() method."""
        ...

    @period_finish.setter
    def period_finish(self, value: typing.Optional[datetime.datetime]) -> None:
        ...

    @property
    def out_of_sample_max_end_date(self) -> typing.Optional[datetime.datetime]:
        """Backtest maximum end date"""
        ...

    @out_of_sample_max_end_date.setter
    def out_of_sample_max_end_date(self, value: typing.Optional[datetime.datetime]) -> None:
        ...

    @property
    def out_of_sample_days(self) -> int:
        """The backtest out of sample day count"""
        ...

    @out_of_sample_days.setter
    def out_of_sample_days(self, value: int) -> None:
        ...

    @property
    def tradeable_dates(self) -> int:
        """Estimated number of trading days in this backtest task based on the start-end dates."""
        ...

    @tradeable_dates.setter
    def tradeable_dates(self, value: int) -> None:
        ...

    @property
    def debugging(self) -> bool:
        """True, if this is a debugging backtest"""
        ...

    @debugging.setter
    def debugging(self, value: bool) -> None:
        ...

    @property
    def cash_amount(self) -> typing.Optional[QuantConnect.Securities.CashAmount]:
        """Optional initial cash amount if set"""
        ...

    @cash_amount.setter
    def cash_amount(self, value: typing.Optional[QuantConnect.Securities.CashAmount]) -> None:
        ...

    @property
    def algorithm_mode(self) -> QuantConnect.AlgorithmMode:
        """Algorithm running mode."""
        ...

    @overload
    def __init__(self) -> None:
        """Default constructor for JSON"""
        ...

    @overload
    def __init__(self, user_id: int, project_id: int, session_id: str, algorithm_data: typing.List[int], starting_capital: float, name: str) -> None:
        """Initialize the backtest task packet."""
        ...

    @overload
    def __init__(self, user_id: int, project_id: int, session_id: str, algorithm_data: typing.List[int], name: str, starting_capital: typing.Optional[QuantConnect.Securities.CashAmount] = None) -> None:
        """Initialize the backtest task packet."""
        ...


class AlgorithmNameUpdatePacket(QuantConnect.Packets.Packet):
    """Packet to communicate updates to the algorithm's name"""

    @property
    def algorithm_id(self) -> str:
        """Algorithm id for this order event"""
        ...

    @algorithm_id.setter
    def algorithm_id(self, value: str) -> None:
        ...

    @property
    def name(self) -> str:
        """The new name"""
        ...

    @name.setter
    def name(self, value: str) -> None:
        ...

    @overload
    def __init__(self) -> None:
        """Default constructor for JSON"""
        ...

    @overload
    def __init__(self, algorithm_id: str, name: str) -> None:
        """Create a new instance of the algorithm tags up[date packet"""
        ...


class LiveNodePacket(QuantConnect.Packets.AlgorithmNodePacket):
    """Live job task packet: container for any live specific job variables"""

    @property
    def deploy_id(self) -> str:
        """Deploy Id for this live algorithm."""
        ...

    @deploy_id.setter
    def deploy_id(self, value: str) -> None:
        ...

    @property
    def brokerage(self) -> str:
        """String name of the brokerage we're trading with"""
        ...

    @brokerage.setter
    def brokerage(self, value: str) -> None:
        ...

    @property
    def brokerage_data(self) -> System.Collections.Generic.Dictionary[str, str]:
        """String-String Dictionary of Brokerage Data for this Live Job"""
        ...

    @brokerage_data.setter
    def brokerage_data(self, value: System.Collections.Generic.Dictionary[str, str]) -> None:
        ...

    @property
    def data_queue_handler(self) -> str:
        """String name of the DataQueueHandler or LiveDataProvider we're running with"""
        ...

    @data_queue_handler.setter
    def data_queue_handler(self, value: str) -> None:
        ...

    @property
    def data_channel_provider(self) -> str:
        """String name of the DataChannelProvider we're running with"""
        ...

    @data_channel_provider.setter
    def data_channel_provider(self, value: str) -> None:
        ...

    @property
    def disable_acknowledgement(self) -> bool:
        """Gets flag indicating whether or not the message should be acknowledged and removed from the queue"""
        ...

    @disable_acknowledgement.setter
    def disable_acknowledgement(self, value: bool) -> None:
        ...

    @property
    def notification_events(self) -> System.Collections.Generic.HashSet[str]:
        """A list of event types to generate notifications for, which will use notification_targets"""
        ...

    @notification_events.setter
    def notification_events(self, value: System.Collections.Generic.HashSet[str]) -> None:
        ...

    @property
    def notification_targets(self) -> typing.List[QuantConnect.Notifications.Notification]:
        """A list of notification targets to use"""
        ...

    @notification_targets.setter
    def notification_targets(self, value: typing.List[QuantConnect.Notifications.Notification]) -> None:
        ...

    @property
    def live_data_types(self) -> System.Collections.Generic.HashSet[str]:
        """List of real time data types available in the live trading environment"""
        ...

    @live_data_types.setter
    def live_data_types(self, value: System.Collections.Generic.HashSet[str]) -> None:
        ...

    @property
    def algorithm_mode(self) -> QuantConnect.AlgorithmMode:
        """Algorithm running mode."""
        ...

    def __init__(self) -> None:
        """Default constructor for JSON of the Live Task Packet"""
        ...


class ResearchNodePacket(QuantConnect.Packets.AlgorithmNodePacket):
    """Represents a research node packet"""

    @property
    def research_id(self) -> str:
        """The research id"""
        ...

    @research_id.setter
    def research_id(self, value: str) -> None:
        ...

    @property
    def research_token(self) -> str:
        """Associated research token"""
        ...

    @research_token.setter
    def research_token(self, value: str) -> None:
        ...

    def __init__(self) -> None:
        """Creates a new instance"""
        ...


class DebugPacket(QuantConnect.Packets.Packet):
    """Send a simple debug message from the users algorithm to the console."""

    @property
    def message(self) -> str:
        """String debug message to send to the users console"""
        ...

    @message.setter
    def message(self, value: str) -> None:
        ...

    @property
    def algorithm_id(self) -> str:
        """Associated algorithm Id."""
        ...

    @algorithm_id.setter
    def algorithm_id(self, value: str) -> None:
        ...

    @property
    def compile_id(self) -> str:
        """Compile id of the algorithm sending this message"""
        ...

    @compile_id.setter
    def compile_id(self, value: str) -> None:
        ...

    @property
    def project_id(self) -> int:
        """Project Id for this message"""
        ...

    @project_id.setter
    def project_id(self, value: int) -> None:
        ...

    @property
    def toast(self) -> bool:
        """
        True to emit message as a popup notification (toast),
        false to emit message in console as text
        """
        ...

    @toast.setter
    def toast(self, value: bool) -> None:
        ...

    @overload
    def __init__(self) -> None:
        """Default constructor for JSON"""
        ...

    @overload
    def __init__(self, packet_type: QuantConnect.Packets.PacketType) -> None:
        """
        Constructor for inherited types
        
        
        This codeEntityType is protected.
        
        :param packet_type: The type of packet to create
        """
        ...

    @overload
    def __init__(self, project_id: int, algorithm_id: str, compile_id: str, message: str, toast: bool = False) -> None:
        """Create a new instance of the notify debug packet:"""
        ...


class AlphaNodePacket(QuantConnect.Packets.LiveNodePacket):
    """Alpha job packet"""

    @property
    def alpha_id(self) -> str:
        """Gets or sets the alpha id"""
        ...

    @alpha_id.setter
    def alpha_id(self, value: str) -> None:
        ...

    def __init__(self) -> None:
        """Initializes a new default instance of the AlgorithmNodePacket class"""
        ...


class BacktestResult(QuantConnect.Result):
    """Backtest results object class - result specific items from the packet."""

    @property
    def rolling_window(self) -> System.Collections.Generic.Dictionary[str, QuantConnect.Statistics.AlgorithmPerformance]:
        """Rolling window detailed statistics."""
        ...

    @rolling_window.setter
    def rolling_window(self, value: System.Collections.Generic.Dictionary[str, QuantConnect.Statistics.AlgorithmPerformance]) -> None:
        ...

    @property
    def total_performance(self) -> QuantConnect.Statistics.AlgorithmPerformance:
        """Rolling window detailed statistics."""
        ...

    @total_performance.setter
    def total_performance(self, value: QuantConnect.Statistics.AlgorithmPerformance) -> None:
        ...

    @overload
    def __init__(self) -> None:
        """Default Constructor"""
        ...

    @overload
    def __init__(self, parameters: QuantConnect.Packets.BacktestResultParameters) -> None:
        """Constructor for the result class using dictionary objects."""
        ...


class BacktestResultPacket(QuantConnect.Packets.Packet):
    """Backtest result packet: send backtest information to GUI for user consumption."""

    @property
    def user_id(self) -> int:
        """User Id placing this task"""
        ...

    @user_id.setter
    def user_id(self, value: int) -> None:
        ...

    @property
    def project_id(self) -> int:
        """Project Id of the this task."""
        ...

    @project_id.setter
    def project_id(self, value: int) -> None:
        ...

    @property
    def session_id(self) -> str:
        """User Session Id"""
        ...

    @session_id.setter
    def session_id(self, value: str) -> None:
        ...

    @property
    def backtest_id(self) -> str:
        """BacktestId for this result packet"""
        ...

    @backtest_id.setter
    def backtest_id(self, value: str) -> None:
        ...

    @property
    def optimization_id(self) -> str:
        """OptimizationId for this result packet if any"""
        ...

    @optimization_id.setter
    def optimization_id(self, value: str) -> None:
        ...

    @property
    def compile_id(self) -> str:
        """Compile Id for the algorithm which generated this result packet."""
        ...

    @compile_id.setter
    def compile_id(self, value: str) -> None:
        ...

    @property
    def period_start(self) -> datetime.datetime:
        """Start of the backtest period as defined in Initialize() method."""
        ...

    @period_start.setter
    def period_start(self, value: datetime.datetime) -> None:
        ...

    @property
    def period_finish(self) -> datetime.datetime:
        """End of the backtest period as defined in the Initialize() method."""
        ...

    @period_finish.setter
    def period_finish(self, value: datetime.datetime) -> None:
        ...

    @property
    def date_requested(self) -> datetime.datetime:
        """DateTime (EST) the user requested this backtest."""
        ...

    @date_requested.setter
    def date_requested(self, value: datetime.datetime) -> None:
        ...

    @property
    def date_finished(self) -> datetime.datetime:
        """DateTime (EST) when the backtest was completed."""
        ...

    @date_finished.setter
    def date_finished(self, value: datetime.datetime) -> None:
        ...

    @property
    def progress(self) -> float:
        """Progress of the backtest as a percentage from 0-1 based on the days lapsed from start-finish."""
        ...

    @progress.setter
    def progress(self, value: float) -> None:
        ...

    @property
    def name(self) -> str:
        """Name of this backtest."""
        ...

    @name.setter
    def name(self, value: str) -> None:
        ...

    @property
    def results(self) -> QuantConnect.Packets.BacktestResult:
        """Result data object for this backtest"""
        ...

    @results.setter
    def results(self, value: QuantConnect.Packets.BacktestResult) -> None:
        ...

    @property
    def processing_time(self) -> float:
        """Processing time of the algorithm (from moment the algorithm arrived on the algorithm node)"""
        ...

    @processing_time.setter
    def processing_time(self, value: float) -> None:
        ...

    @property
    def tradeable_dates(self) -> int:
        """Estimated number of tradeable days in the backtest based on the start and end date or the backtest"""
        ...

    @tradeable_dates.setter
    def tradeable_dates(self, value: int) -> None:
        ...

    @overload
    def __init__(self) -> None:
        """Default constructor for JSON Serialization"""
        ...

    @overload
    def __init__(self, json: str) -> None:
        """Compose the packet from a JSON string:"""
        ...

    @overload
    def __init__(self, job: QuantConnect.Packets.BacktestNodePacket, results: QuantConnect.Packets.BacktestResult, end_date: typing.Union[datetime.datetime, datetime.date], start_date: typing.Union[datetime.datetime, datetime.date], progress: float = 1) -> None:
        """
        Compose result data packet - with tradable dates from the backtest job task and the partial result packet.
        
        :param job: Job that started this request
        :param results: Results class for the Backtest job
        :param end_date: The algorithms backtest end date
        :param start_date: The algorithms backtest start date
        :param progress: Progress of the packet. For the packet we assume progess of 100%.
        """
        ...

    @staticmethod
    def create_empty(job: QuantConnect.Packets.BacktestNodePacket) -> QuantConnect.Packets.BacktestResultPacket:
        """
        Creates an empty result packet, useful when the algorithm fails to initialize
        
        :param job: The associated job packet
        :returns: An empty result packet.
        """
        ...


class LiveResultParameters(QuantConnect.Packets.BaseResultParameters):
    """Defines the parameters for LiveResult"""

    @property
    def holdings(self) -> System.Collections.Generic.IDictionary[str, QuantConnect.Holding]:
        """Holdings dictionary of algorithm holdings information"""
        ...

    @holdings.setter
    def holdings(self, value: System.Collections.Generic.IDictionary[str, QuantConnect.Holding]) -> None:
        ...

    @property
    def cash_book(self) -> QuantConnect.Securities.CashBook:
        """Cashbook for the algorithm's live results."""
        ...

    @cash_book.setter
    def cash_book(self, value: QuantConnect.Securities.CashBook) -> None:
        ...

    @property
    def server_statistics(self) -> System.Collections.Generic.IDictionary[str, str]:
        """Server status information, including CPU/RAM usage, ect..."""
        ...

    @server_statistics.setter
    def server_statistics(self, value: System.Collections.Generic.IDictionary[str, str]) -> None:
        ...

    def __init__(self, charts: System.Collections.Generic.IDictionary[str, QuantConnect.Chart], orders: System.Collections.Generic.IDictionary[int, QuantConnect.Orders.Order], profit_loss: System.Collections.Generic.IDictionary[datetime.datetime, float], holdings: System.Collections.Generic.IDictionary[str, QuantConnect.Holding], cash_book: QuantConnect.Securities.CashBook, statistics: System.Collections.Generic.IDictionary[str, str], runtime_statistics: System.Collections.Generic.IDictionary[str, str], order_events: typing.List[QuantConnect.Orders.OrderEvent], server_statistics: System.Collections.Generic.IDictionary[str, str] = None, algorithm_configuration: QuantConnect.AlgorithmConfiguration = None, state: System.Collections.Generic.IDictionary[str, str] = None) -> None:
        """Creates a new instance"""
        ...


class SystemDebugPacket(QuantConnect.Packets.DebugPacket):
    """Debug packets generated by Lean"""

    @overload
    def __init__(self) -> None:
        """Default constructor for JSON"""
        ...

    @overload
    def __init__(self, project_id: int, algorithm_id: str, compile_id: str, message: str, toast: bool = False) -> None:
        """Create a new instance of the system debug packet"""
        ...


class HistoryRequest(System.Object):
    """
    Specifies request parameters for a single historical request.
    A HistoryPacket is made of multiple requests for data. These
    are used to request data during live mode from a data server
    """

    @property
    def start_time_utc(self) -> datetime.datetime:
        """The start time to request data in UTC"""
        ...

    @start_time_utc.setter
    def start_time_utc(self, value: datetime.datetime) -> None:
        ...

    @property
    def end_time_utc(self) -> datetime.datetime:
        """The end time to request data in UTC"""
        ...

    @end_time_utc.setter
    def end_time_utc(self, value: datetime.datetime) -> None:
        ...

    @property
    def symbol(self) -> QuantConnect.Symbol:
        """The symbol to request data for"""
        ...

    @symbol.setter
    def symbol(self, value: QuantConnect.Symbol) -> None:
        ...

    @property
    def resolution(self) -> QuantConnect.Resolution:
        """The requested resolution"""
        ...

    @resolution.setter
    def resolution(self, value: QuantConnect.Resolution) -> None:
        ...

    @property
    def tick_type(self) -> QuantConnect.TickType:
        """The type of data to retrieve"""
        ...

    @tick_type.setter
    def tick_type(self, value: QuantConnect.TickType) -> None:
        ...


class HistoryPacket(QuantConnect.Packets.Packet):
    """Packet for history jobs"""

    @property
    def queue_name(self) -> str:
        """The queue where the data should be sent"""
        ...

    @queue_name.setter
    def queue_name(self, value: str) -> None:
        ...

    @property
    def requests(self) -> typing.List[QuantConnect.Packets.HistoryRequest]:
        """The individual requests to be processed"""
        ...

    @requests.setter
    def requests(self, value: typing.List[QuantConnect.Packets.HistoryRequest]) -> None:
        ...

    def __init__(self) -> None:
        """Initializes a new instance of the HistoryPacket class"""
        ...


class HistoryResultType(IntEnum):
    """Specifies various types of history results"""

    FILE = 0
    """The requested file data"""

    STATUS = 1
    """The request's status"""

    COMPLETED = 2
    """The request is completed"""

    ERROR = 3
    """The request had an error"""


class HistoryResult(System.Object, metaclass=abc.ABCMeta):
    """
    Provides a container for results from history requests. This contains
    the file path relative to the /Data folder where the data can be written
    """

    @property
    def type(self) -> QuantConnect.Packets.HistoryResultType:
        """Gets the type of history result"""
        ...

    def __init__(self, type: QuantConnect.Packets.HistoryResultType) -> None:
        """
        Initializes a new instance of the HistoryResult class
        
        
        This codeEntityType is protected.
        
        :param type: The type of history result
        """
        ...


class FileHistoryResult(QuantConnect.Packets.HistoryResult):
    """Defines requested file data for a history request"""

    @property
    def filepath(self) -> str:
        """The relative file path where the data should be written"""
        ...

    @filepath.setter
    def filepath(self, value: str) -> None:
        ...

    @property
    def file(self) -> typing.List[int]:
        """The file's contents, this is a zipped csv file"""
        ...

    @file.setter
    def file(self, value: typing.List[int]) -> None:
        ...

    @overload
    def __init__(self) -> None:
        """Default constructor for serializers"""
        ...

    @overload
    def __init__(self, filepath: str, file: typing.List[int]) -> None:
        """
        Initializes a new instance of the HistoryResult class
        
        :param filepath: The relative file path where the file should be written, rooted in /Data, so for example ./forex/fxcm/daily/eurusd.zip
        :param file: The zipped csv file content in bytes
        """
        ...


class CompletedHistoryResult(QuantConnect.Packets.HistoryResult):
    """Specifies the completed message from a history result"""

    def __init__(self) -> None:
        """Initializes a new instance of CompletedHistoryResult class"""
        ...


class ErrorHistoryResult(QuantConnect.Packets.HistoryResult):
    """Specfies an error message in a history result"""

    @property
    def message(self) -> str:
        """Gets the error that was encountered"""
        ...

    @message.setter
    def message(self, value: str) -> None:
        ...

    @overload
    def __init__(self) -> None:
        """Default constructor for serializers"""
        ...

    @overload
    def __init__(self, message: str) -> None:
        """
        Initializes a new instance of the ErrorHistoryResult class
        
        :param message: The error message
        """
        ...


class StatusHistoryResult(QuantConnect.Packets.HistoryResult):
    """Specifies the progress of a request"""

    @property
    def progress(self) -> int:
        """Gets the progress of the request"""
        ...

    @progress.setter
    def progress(self, value: int) -> None:
        ...

    @overload
    def __init__(self) -> None:
        """Default constructor for serializers"""
        ...

    @overload
    def __init__(self, progress: int) -> None:
        """
        Initializes a new instance of the StatusHistoryResult class
        
        :param progress: The progress, from 0 to 100
        """
        ...


class AlgorithmStatusPacket(QuantConnect.Packets.Packet):
    """Algorithm status update information packet"""

    @property
    def status(self) -> QuantConnect.AlgorithmStatus:
        """Current algorithm status"""
        ...

    @status.setter
    def status(self, value: QuantConnect.AlgorithmStatus) -> None:
        ...

    @property
    def chart_subscription(self) -> str:
        """Chart we're subscribed to for live trading."""
        ...

    @chart_subscription.setter
    def chart_subscription(self, value: str) -> None:
        ...

    @property
    def message(self) -> str:
        """Optional message or reason for state change."""
        ...

    @message.setter
    def message(self, value: str) -> None:
        ...

    @property
    def algorithm_id(self) -> str:
        """Algorithm Id associated with this status packet"""
        ...

    @algorithm_id.setter
    def algorithm_id(self, value: str) -> None:
        ...

    @property
    def optimization_id(self) -> str:
        """OptimizationId for this result packet if any"""
        ...

    @optimization_id.setter
    def optimization_id(self, value: str) -> None:
        ...

    @property
    def project_id(self) -> int:
        """Project Id associated with this status packet"""
        ...

    @project_id.setter
    def project_id(self, value: int) -> None:
        ...

    @property
    def channel_status(self) -> str:
        """The current state of the channel"""
        ...

    @channel_status.setter
    def channel_status(self, value: str) -> None:
        ...

    @overload
    def __init__(self) -> None:
        """Default constructor for JSON"""
        ...

    @overload
    def __init__(self, algorithm_id: str, project_id: int, status: QuantConnect.AlgorithmStatus, message: str = ...) -> None:
        """Initialize algorithm state packet:"""
        ...


class RuntimeErrorPacket(QuantConnect.Packets.Packet):
    """
    Algorithm runtime error packet from the lean engine.
    This is a managed error which stops the algorithm execution.
    """

    @property
    def message(self) -> str:
        """Runtime error message from the exception"""
        ...

    @message.setter
    def message(self, value: str) -> None:
        ...

    @property
    def algorithm_id(self) -> str:
        """Algorithm id which generated this runtime error"""
        ...

    @algorithm_id.setter
    def algorithm_id(self, value: str) -> None:
        ...

    @property
    def stack_trace(self) -> str:
        """Error stack trace information string passed through from the Lean exception"""
        ...

    @stack_trace.setter
    def stack_trace(self, value: str) -> None:
        ...

    @property
    def user_id(self) -> int:
        """User Id associated with the backtest that threw the error"""
        ...

    @user_id.setter
    def user_id(self, value: int) -> None:
        ...

    @overload
    def __init__(self) -> None:
        """Default constructor for JSON"""
        ...

    @overload
    def __init__(self, user_id: int, algorithm_id: str, message: str, stacktrace: str = ...) -> None:
        """Create a new runtime error packet"""
        ...


class SecurityTypesPacket(QuantConnect.Packets.Packet):
    """Security types packet contains information on the markets the user data has requested."""

    @property
    def types(self) -> typing.List[QuantConnect.SecurityType]:
        """List of Security Type the user has requested (Equity, Forex, Futures etc)."""
        ...

    @types.setter
    def types(self, value: typing.List[QuantConnect.SecurityType]) -> None:
        ...

    @property
    def types_csv(self) -> str:
        """CSV formatted, lower case list of SecurityTypes for the web API."""
        ...

    def __init__(self) -> None:
        """Default constructor for JSON"""
        ...


class AlphaResultPacket(QuantConnect.Packets.Packet):
    """Provides a packet type for transmitting alpha insights data"""

    @property
    def user_id(self) -> int:
        """The user's id that deployed the alpha stream"""
        ...

    @user_id.setter
    def user_id(self, value: int) -> None:
        ...

    @property
    def alpha_id(self) -> str:
        """
        The deployed alpha id. This is the id generated upon submssion to the alpha marketplace.
        If this is a user backtest or live algo then this will not be specified
        """
        ...

    @alpha_id.setter
    def alpha_id(self, value: str) -> None:
        ...

    @property
    def algorithm_id(self) -> str:
        """The algorithm's unique identifier"""
        ...

    @algorithm_id.setter
    def algorithm_id(self, value: str) -> None:
        ...

    @property
    def insights(self) -> typing.List[QuantConnect.Algorithm.Framework.Alphas.Insight]:
        """The generated insights"""
        ...

    @insights.setter
    def insights(self, value: typing.List[QuantConnect.Algorithm.Framework.Alphas.Insight]) -> None:
        ...

    @property
    def order_events(self) -> typing.List[QuantConnect.Orders.OrderEvent]:
        """The generated OrderEvents"""
        ...

    @order_events.setter
    def order_events(self, value: typing.List[QuantConnect.Orders.OrderEvent]) -> None:
        ...

    @property
    def orders(self) -> typing.List[QuantConnect.Orders.Order]:
        """The new or updated Orders"""
        ...

    @orders.setter
    def orders(self, value: typing.List[QuantConnect.Orders.Order]) -> None:
        ...

    @overload
    def __init__(self) -> None:
        """Initializes a new instance of the AlphaResultPacket class"""
        ...

    @overload
    def __init__(self, algorithm_id: str, user_id: int, insights: typing.List[QuantConnect.Algorithm.Framework.Alphas.Insight] = None, order_events: typing.List[QuantConnect.Orders.OrderEvent] = None, orders: typing.List[QuantConnect.Orders.Order] = None) -> None:
        """
        Initializes a new instance of the AlphaResultPacket class
        
        :param algorithm_id: The algorithm's unique identifier
        :param user_id: The user's id
        :param insights: Alphas generated by the algorithm
        :param order_events: OrderEvents generated by the algorithm
        :param orders: Orders generated or updated by the algorithm
        """
        ...


class AlgorithmTagsUpdatePacket(QuantConnect.Packets.Packet):
    """Packet to communicate updates to the algorithm tags"""

    @property
    def algorithm_id(self) -> str:
        """Algorithm id for this order event"""
        ...

    @algorithm_id.setter
    def algorithm_id(self, value: str) -> None:
        ...

    @property
    def tags(self) -> System.Collections.Generic.HashSet[str]:
        """The new tags"""
        ...

    @tags.setter
    def tags(self, value: System.Collections.Generic.HashSet[str]) -> None:
        ...

    @overload
    def __init__(self) -> None:
        """Default constructor for JSON"""
        ...

    @overload
    def __init__(self, algorithm_id: str, tags: System.Collections.Generic.HashSet[str]) -> None:
        """Create a new instance of the algorithm tags up[date packet"""
        ...


class LiveResult(QuantConnect.Result):
    """Live results object class for packaging live result data."""

    @property
    def holdings(self) -> System.Collections.Generic.IDictionary[str, QuantConnect.Holding]:
        """Holdings dictionary of algorithm holdings information"""
        ...

    @holdings.setter
    def holdings(self, value: System.Collections.Generic.IDictionary[str, QuantConnect.Holding]) -> None:
        ...

    @property
    def cash_book(self) -> QuantConnect.Securities.CashBook:
        """Cashbook for the algorithm's live results."""
        ...

    @cash_book.setter
    def cash_book(self, value: QuantConnect.Securities.CashBook) -> None:
        ...

    @property
    def cash(self) -> System.Collections.Generic.Dictionary[str, QuantConnect.Securities.Cash]:
        """Cash for the algorithm's live results."""
        ...

    @cash.setter
    def cash(self, value: System.Collections.Generic.Dictionary[str, QuantConnect.Securities.Cash]) -> None:
        ...

    @property
    def account_currency(self) -> str:
        """The algorithm's account currency"""
        ...

    @account_currency.setter
    def account_currency(self, value: str) -> None:
        ...

    @property
    def account_currency_symbol(self) -> str:
        """The algorithm's account currency"""
        ...

    @account_currency_symbol.setter
    def account_currency_symbol(self, value: str) -> None:
        ...

    @overload
    def __init__(self) -> None:
        """Default Constructor"""
        ...

    @overload
    def __init__(self, parameters: QuantConnect.Packets.LiveResultParameters) -> None:
        """Constructor for the result class for dictionary objects"""
        ...


class LiveResultPacket(QuantConnect.Packets.Packet):
    """Live result packet from a lean engine algorithm."""

    @property
    def user_id(self) -> int:
        """User Id sending result packet"""
        ...

    @user_id.setter
    def user_id(self, value: int) -> None:
        ...

    @property
    def project_id(self) -> int:
        """Project Id of the result packet"""
        ...

    @project_id.setter
    def project_id(self, value: int) -> None:
        ...

    @property
    def deploy_id(self) -> str:
        """Live Algorithm Id (DeployId) for this result packet"""
        ...

    @deploy_id.setter
    def deploy_id(self, value: str) -> None:
        ...

    @property
    def results(self) -> QuantConnect.Packets.LiveResult:
        """Result data object for this result packet"""
        ...

    @results.setter
    def results(self, value: QuantConnect.Packets.LiveResult) -> None:
        ...

    @overload
    def __init__(self) -> None:
        """Default constructor for JSON Serialization"""
        ...

    @overload
    def __init__(self, json: str) -> None:
        """Compose the packet from a JSON string:"""
        ...

    @overload
    def __init__(self, job: QuantConnect.Packets.LiveNodePacket, results: QuantConnect.Packets.LiveResult) -> None:
        """
        Compose Live Result Data Packet - With tradable dates
        
        :param job: Job that started this request
        :param results: Results class for the Backtest job
        """
        ...

    @staticmethod
    def create_empty(job: QuantConnect.Packets.LiveNodePacket) -> QuantConnect.Packets.LiveResultPacket:
        """
        Creates an empty result packet, useful when the algorithm fails to initialize
        
        :param job: The associated job packet
        :returns: An empty result packet.
        """
        ...


class MarketHours(System.Object):
    """Market open hours model for pre, normal and post market hour definitions."""

    @property
    def start(self) -> datetime.datetime:
        """Start time for this market hour category"""
        ...

    @start.setter
    def start(self, value: datetime.datetime) -> None:
        ...

    @property
    def end(self) -> datetime.datetime:
        """End time for this market hour category"""
        ...

    @end.setter
    def end(self, value: datetime.datetime) -> None:
        ...

    def __init__(self, reference_date: typing.Union[datetime.datetime, datetime.date], default_start: float, default_end: float) -> None:
        """
        Market hours initializer given an hours since midnight measure for the market hours today
        
        :param reference_date: Reference date used for as base date from the specified hour offsets
        :param default_start: Time in hours since midnight to start this open period.
        :param default_end: Time in hours since midnight to end this open period.
        """
        ...


class MarketToday(System.Object):
    """Market today information class"""

    @property
    def date(self) -> datetime.datetime:
        """Date this packet was generated."""
        ...

    @date.setter
    def date(self, value: datetime.datetime) -> None:
        ...

    @property
    def status(self) -> str:
        """Given the dates and times above, what is the current market status - open or closed."""
        ...

    @status.setter
    def status(self, value: str) -> None:
        ...

    @property
    def pre_market(self) -> QuantConnect.Packets.MarketHours:
        """Premarket hours for today"""
        ...

    @pre_market.setter
    def pre_market(self, value: QuantConnect.Packets.MarketHours) -> None:
        ...

    @property
    def open(self) -> QuantConnect.Packets.MarketHours:
        """Normal trading market hours for today"""
        ...

    @open.setter
    def open(self, value: QuantConnect.Packets.MarketHours) -> None:
        ...

    @property
    def post_market(self) -> QuantConnect.Packets.MarketHours:
        """Post market hours for today"""
        ...

    @post_market.setter
    def post_market(self, value: QuantConnect.Packets.MarketHours) -> None:
        ...

    def __init__(self) -> None:
        """Default constructor (required for JSON serialization)"""
        ...


class LogPacket(QuantConnect.Packets.Packet):
    """Simple log message instruction from the lean engine."""

    @property
    def message(self) -> str:
        """Log message to the users console:"""
        ...

    @message.setter
    def message(self, value: str) -> None:
        ...

    @property
    def algorithm_id(self) -> str:
        """Algorithm Id requesting this logging"""
        ...

    @algorithm_id.setter
    def algorithm_id(self, value: str) -> None:
        ...

    @overload
    def __init__(self) -> None:
        """Default constructor for JSON"""
        ...

    @overload
    def __init__(self, algorithm_id: str, message: str) -> None:
        """Create a new instance of the notify Log packet:"""
        ...


class HandledErrorPacket(QuantConnect.Packets.Packet):
    """
    Algorithm runtime error packet from the lean engine.
    This is a managed error which stops the algorithm execution.
    """

    @property
    def message(self) -> str:
        """Runtime error message from the exception"""
        ...

    @message.setter
    def message(self, value: str) -> None:
        ...

    @property
    def algorithm_id(self) -> str:
        """Algorithm id which generated this runtime error"""
        ...

    @algorithm_id.setter
    def algorithm_id(self, value: str) -> None:
        ...

    @property
    def stack_trace(self) -> str:
        """Error stack trace information string passed through from the Lean exception"""
        ...

    @stack_trace.setter
    def stack_trace(self, value: str) -> None:
        ...

    @overload
    def __init__(self) -> None:
        """Default constructor for JSON"""
        ...

    @overload
    def __init__(self, algorithm_id: str, message: str, stacktrace: str = ...) -> None:
        """Create a new handled error packet"""
        ...


class OrderEventPacket(QuantConnect.Packets.Packet):
    """Order event packet for passing updates on the state of an order to the portfolio."""

    @property
    def event(self) -> QuantConnect.Orders.OrderEvent:
        """Order event object"""
        ...

    @event.setter
    def event(self, value: QuantConnect.Orders.OrderEvent) -> None:
        ...

    @property
    def algorithm_id(self) -> str:
        """Algorithm id for this order event"""
        ...

    @algorithm_id.setter
    def algorithm_id(self, value: str) -> None:
        ...

    @overload
    def __init__(self) -> None:
        """Default constructor for JSON"""
        ...

    @overload
    def __init__(self, algorithm_id: str, event_order: QuantConnect.Orders.OrderEvent) -> None:
        """Create a new instance of the order event packet"""
        ...





from typing import overload
from enum import IntEnum
import typing

import QuantConnect
import QuantConnect.DownloaderDataProvider.Launcher
import QuantConnect.DownloaderDataProvider.Launcher.Models
import QuantConnect.Interfaces
import System
import System.Collections.Generic


class DownloaderDataProviderArgumentParser(System.Object):
    """This class has no documentation."""

    @staticmethod
    def parse_arguments(args: typing.List[str]) -> System.Collections.Generic.Dictionary[str, System.Object]:
        """
        Parses the command-line arguments and returns a dictionary containing parsed values.
        
        :param args: An array of command-line arguments.
        :returns: A dictionary containing parsed values from the command-line arguments.
        """
        ...


class Program(System.Object):
    """This class has no documentation."""

    @staticmethod
    def initialize_configurations() -> None:
        """
        Initializes various configurations for the application.
        This method sets up logging, data providers, map file providers, and factor file providers.
        """
        ...

    @staticmethod
    def main(args: typing.List[str]) -> None:
        """
        The main entry point for the application.
        
        :param args: Command-line arguments passed to the application.
        """
        ...

    @staticmethod
    def run_download(data_downloader: QuantConnect.IDataDownloader, data_download_config: QuantConnect.DownloaderDataProvider.Launcher.Models.DataDownloadConfig, data_directory: str, data_cache_provider: QuantConnect.Interfaces.IDataCacheProvider, map_symbol: bool = True) -> None:
        """
        Executes a data download operation using the specified data downloader.
        
        :param data_downloader: An instance of an object implementing the IDataDownloader interface, responsible for downloading data.
        :param data_download_config: Configuration settings for the data download operation.
        :param data_directory: The directory where the downloaded data will be stored.
        :param data_cache_provider: The provider used to cache history data files
        :param map_symbol: True if the symbol should be mapped while writing the data
        """
        ...





from typing import overload
from enum import IntEnum
import datetime
import typing

import System
import System.ComponentModel
import System.Timers

System_Timers__EventContainer_Callable = typing.TypeVar("System_Timers__EventContainer_Callable")
System_Timers__EventContainer_ReturnType = typing.TypeVar("System_Timers__EventContainer_ReturnType")


class ElapsedEventArgs(System.EventArgs):
    """This class has no documentation."""

    @property
    def signal_time(self) -> datetime.datetime:
        ...

    def __init__(self, signal_time: typing.Union[datetime.datetime, datetime.date]) -> None:
        ...


class Timer(System.ComponentModel.Component, System.ComponentModel.ISupportInitialize):
    """This class has no documentation."""

    @property
    def auto_reset(self) -> bool:
        ...

    @auto_reset.setter
    def auto_reset(self, value: bool) -> None:
        ...

    @property
    def enabled(self) -> bool:
        ...

    @enabled.setter
    def enabled(self, value: bool) -> None:
        ...

    @property
    def interval(self) -> float:
        ...

    @interval.setter
    def interval(self, value: float) -> None:
        ...

    @property
    def elapsed(self) -> _EventContainer[typing.Callable[[System.Object, System.Timers.ElapsedEventArgs], typing.Any], typing.Any]:
        ...

    @elapsed.setter
    def elapsed(self, value: _EventContainer[typing.Callable[[System.Object, System.Timers.ElapsedEventArgs], typing.Any], typing.Any]) -> None:
        ...

    @property
    def site(self) -> System.ComponentModel.ISite:
        ...

    @site.setter
    def site(self, value: System.ComponentModel.ISite) -> None:
        ...

    @property
    def synchronizing_object(self) -> System.ComponentModel.ISynchronizeInvoke:
        ...

    @synchronizing_object.setter
    def synchronizing_object(self, value: System.ComponentModel.ISynchronizeInvoke) -> None:
        ...

    @overload
    def __init__(self) -> None:
        ...

    @overload
    def __init__(self, interval: float) -> None:
        ...

    @overload
    def __init__(self, interval: datetime.timedelta) -> None:
        ...

    def begin_init(self) -> None:
        ...

    def close(self) -> None:
        ...

    def dispose(self, disposing: bool) -> None:
        ...

    def end_init(self) -> None:
        ...

    def start(self) -> None:
        ...

    def stop(self) -> None:
        ...


class TimersDescriptionAttribute(System.ComponentModel.DescriptionAttribute):
    """This class has no documentation."""

    @property
    def description(self) -> str:
        ...

    def __init__(self, description: str) -> None:
        ...


class _EventContainer(typing.Generic[System_Timers__EventContainer_Callable, System_Timers__EventContainer_ReturnType]):
    """This class is used to provide accurate autocomplete on events and cannot be imported."""

    def __call__(self, *args: typing.Any, **kwargs: typing.Any) -> System_Timers__EventContainer_ReturnType:
        """Fires the event."""
        ...

    def __iadd__(self, item: System_Timers__EventContainer_Callable) -> typing.Self:
        """Registers an event handler."""
        ...

    def __isub__(self, item: System_Timers__EventContainer_Callable) -> typing.Self:
        """Unregisters an event handler."""
        ...





from typing import overload
from enum import IntEnum
import abc
import typing

import QuantConnect
import QuantConnect.Packets
import QuantConnect.Report.ReportElements
import System


class ReportElement(System.Object, QuantConnect.Report.ReportElements.IReportElement, metaclass=abc.ABCMeta):
    """Common interface for template elements of the report"""

    @property
    def name(self) -> str:
        """Name of this report element"""
        ...

    @name.setter
    def name(self, value: str) -> None:
        ...

    @property
    def key(self) -> str:
        """Template key code."""
        ...

    @key.setter
    def key(self, value: str) -> None:
        ...

    @property
    def json_key(self) -> str:
        """Normalizes the key into a JSON-friendly key"""
        ...

    @property
    def result(self) -> System.Object:
        """Result of the render as an object for serialization to JSON"""
        ...

    @result.setter
    def result(self, value: System.Object) -> None:
        ...

    def render(self) -> str:
        """The generated output string to be injected"""
        ...


class SharpeRatioReportElement(QuantConnect.Report.ReportElements.ReportElement):
    """Class for render the Sharpe Ratio statistic for a report"""

    @property
    def live_result(self) -> QuantConnect.Packets.LiveResult:
        """
        Live result object
        
        
        This codeEntityType is protected.
        """
        ...

    @property
    def backtest_result(self) -> QuantConnect.Packets.BacktestResult:
        """
        Backtest result object
        
        
        This codeEntityType is protected.
        """
        ...

    @property
    def backtest_result_value(self) -> typing.Optional[float]:
        """Sharpe Ratio from a backtest"""
        ...

    def __init__(self, name: str, key: str, backtest: QuantConnect.Packets.BacktestResult, live: QuantConnect.Packets.LiveResult, trading_days_per_year: int) -> None:
        """
        Estimate the sharpe ratio of the strategy.
        
        :param name: Name of the widget
        :param key: Location of injection
        :param backtest: Backtest result object
        :param live: Live result object
        :param trading_days_per_year: The number of trading days per year to get better result of statistics
        """
        ...

    def get_annual_standard_deviation(self, trailing_performance: typing.List[float], trading_days_per_year: float) -> float:
        """
        Get annual standard deviation
        
        :param trailing_performance: The performance for the last period
        :param trading_days_per_year: The number of trading days per year to get better result of statistics
        :returns: Annual standard deviation.
        """
        ...

    def render(self) -> str:
        """The generated output string to be injected"""
        ...


class EstimatedCapacityReportElement(QuantConnect.Report.ReportElements.ReportElement):
    """Capacity Estimation Report Element"""

    def __init__(self, name: str, key: str, backtest: QuantConnect.Packets.BacktestResult, live: QuantConnect.Packets.LiveResult) -> None:
        """
        Create a new capacity estimate
        
        :param name: Name of the widget
        :param key: Location of injection
        :param backtest: Backtest result object
        :param live: Live result object
        """
        ...

    def render(self) -> str:
        """Render element"""
        ...


class ParametersReportElement(QuantConnect.Report.ReportElements.ReportElement):
    """Class for creating a two column table for the Algorithm's Parameters in a report"""

    def __init__(self, name: str, key: str, backtest_configuration: QuantConnect.AlgorithmConfiguration, live_configuration: QuantConnect.AlgorithmConfiguration, template: str) -> None:
        """
        Creates a two column table for the Algorithm's Parameters
        
        :param name: Name of the widget
        :param key: Location of injection
        :param backtest_configuration: The configuration of the backtest algorithm
        :param live_configuration: The configuration of the live algorithm
        :param template: HTML template to use
        """
        ...

    def render(self) -> str:
        """
        Generates a HTML two column table for the Algorithm's Parameters
        
        :returns: Returns a string representing a HTML two column table.
        """
        ...





from typing import overload
from enum import IntEnum
import datetime
import typing

import System
import System.Collections.Generic
import System.Diagnostics
import System.Reflection


class StackFrame(System.Object):
    """This class has no documentation."""

    OFFSET_UNKNOWN: int = -1

    @overload
    def __init__(self) -> None:
        ...

    @overload
    def __init__(self, need_file_info: bool) -> None:
        ...

    @overload
    def __init__(self, skip_frames: int) -> None:
        ...

    @overload
    def __init__(self, skip_frames: int, need_file_info: bool) -> None:
        ...

    @overload
    def __init__(self, file_name: str, line_number: int) -> None:
        ...

    @overload
    def __init__(self, file_name: str, line_number: int, col_number: int) -> None:
        ...

    def get_file_column_number(self) -> int:
        ...

    def get_file_line_number(self) -> int:
        ...

    def get_file_name(self) -> str:
        ...

    def get_il_offset(self) -> int:
        ...

    def get_method(self) -> System.Reflection.MethodBase:
        ...

    def get_native_offset(self) -> int:
        ...

    def to_string(self) -> str:
        ...


class DiagnosticMethodInfo(System.Object):
    """This class has no documentation."""

    @property
    def name(self) -> str:
        ...

    @property
    def declaring_type_name(self) -> str:
        ...

    @property
    def declaring_assembly_name(self) -> str:
        ...

    @staticmethod
    @overload
    def create(delegate: System.Delegate) -> System.Diagnostics.DiagnosticMethodInfo:
        ...

    @staticmethod
    @overload
    def create(frame: System.Diagnostics.StackFrame) -> System.Diagnostics.DiagnosticMethodInfo:
        ...


class DebugProvider(System.Object):
    """This class has no documentation."""

    def fail(self, message: str, detail_message: str) -> None:
        ...

    @staticmethod
    def fail_core(stack_trace: str, message: str, detail_message: str, error_source: str) -> None:
        ...

    def on_indent_level_changed(self, indent_level: int) -> None:
        ...

    def on_indent_size_changed(self, indent_size: int) -> None:
        ...

    def write(self, message: str) -> None:
        ...

    @staticmethod
    def write_core(message: str) -> None:
        ...

    def write_line(self, message: str) -> None:
        ...


class Debug(System.Object):
    """This class has no documentation."""

    class AssertInterpolatedStringHandler:
        """This class has no documentation."""

        def __init__(self, literal_length: int, formatted_count: int, condition: bool, should_append: typing.Optional[bool]) -> typing.Tuple[None, bool]:
            ...

        @overload
        def append_formatted(self, value: typing.Any, alignment: int = 0, format: str = None) -> None:
            ...

        @overload
        def append_formatted(self, value: System.ReadOnlySpan[str]) -> None:
            ...

        @overload
        def append_formatted(self, value: System.ReadOnlySpan[str], alignment: int = 0, format: str = None) -> None:
            ...

        @overload
        def append_formatted(self, value: str) -> None:
            ...

        @overload
        def append_formatted(self, value: str, alignment: int = 0, format: str = None) -> None:
            ...

        def append_literal(self, value: str) -> None:
            ...

    class WriteIfInterpolatedStringHandler:
        """This class has no documentation."""

        def __init__(self, literal_length: int, formatted_count: int, condition: bool, should_append: typing.Optional[bool]) -> typing.Tuple[None, bool]:
            ...

        @overload
        def append_formatted(self, value: typing.Any, alignment: int = 0, format: str = None) -> None:
            ...

        @overload
        def append_formatted(self, value: System.ReadOnlySpan[str]) -> None:
            ...

        @overload
        def append_formatted(self, value: System.ReadOnlySpan[str], alignment: int = 0, format: str = None) -> None:
            ...

        @overload
        def append_formatted(self, value: str) -> None:
            ...

        @overload
        def append_formatted(self, value: str, alignment: int = 0, format: str = None) -> None:
            ...

        def append_literal(self, value: str) -> None:
            ...

    auto_flush: bool

    indent_level: int

    indent_size: int

    @staticmethod
    @overload
    def Assert(condition: bool) -> None:
        ...

    @staticmethod
    @overload
    def Assert(condition: bool, message: str = None) -> None:
        ...

    @staticmethod
    @overload
    def Assert(condition: bool, message: System.Diagnostics.Debug.AssertInterpolatedStringHandler) -> None:
        ...

    @staticmethod
    @overload
    def Assert(condition: bool, message: str, detailMessage: str) -> None:
        ...

    @staticmethod
    @overload
    def Assert(condition: bool, message: System.Diagnostics.Debug.AssertInterpolatedStringHandler, detailMessage: System.Diagnostics.Debug.AssertInterpolatedStringHandler) -> None:
        ...

    @staticmethod
    @overload
    def Assert(condition: bool, message: str, detailMessageFormat: str, *args: typing.Union[System.Object, typing.Iterable[System.Object]]) -> None:
        ...

    @staticmethod
    def close() -> None:
        ...

    @staticmethod
    @overload
    def fail(message: str) -> None:
        ...

    @staticmethod
    @overload
    def fail(message: str, detail_message: str) -> None:
        ...

    @staticmethod
    def flush() -> None:
        ...

    @staticmethod
    def indent() -> None:
        ...

    @staticmethod
    @overload
    def print(message: str) -> None:
        ...

    @staticmethod
    @overload
    def print(format: str, *args: typing.Union[System.Object, typing.Iterable[System.Object]]) -> None:
        ...

    @staticmethod
    def set_provider(provider: System.Diagnostics.DebugProvider) -> System.Diagnostics.DebugProvider:
        ...

    @staticmethod
    def unindent() -> None:
        ...

    @staticmethod
    @overload
    def write(value: typing.Any) -> None:
        ...

    @staticmethod
    @overload
    def write(value: typing.Any, category: str) -> None:
        ...

    @staticmethod
    @overload
    def write(message: str) -> None:
        ...

    @staticmethod
    @overload
    def write(message: str, category: str) -> None:
        ...

    @staticmethod
    @overload
    def write_if(condition: bool, value: typing.Any) -> None:
        ...

    @staticmethod
    @overload
    def write_if(condition: bool, value: typing.Any, category: str) -> None:
        ...

    @staticmethod
    @overload
    def write_if(condition: bool, message: str) -> None:
        ...

    @staticmethod
    @overload
    def write_if(condition: bool, message: System.Diagnostics.Debug.WriteIfInterpolatedStringHandler) -> None:
        ...

    @staticmethod
    @overload
    def write_if(condition: bool, message: str, category: str) -> None:
        ...

    @staticmethod
    @overload
    def write_if(condition: bool, message: System.Diagnostics.Debug.WriteIfInterpolatedStringHandler, category: str) -> None:
        ...

    @staticmethod
    @overload
    def write_line(value: typing.Any) -> None:
        ...

    @staticmethod
    @overload
    def write_line(value: typing.Any, category: str) -> None:
        ...

    @staticmethod
    @overload
    def write_line(message: str) -> None:
        ...

    @staticmethod
    @overload
    def write_line(format: str, *args: typing.Union[System.Object, typing.Iterable[System.Object]]) -> None:
        ...

    @staticmethod
    @overload
    def write_line(message: str, category: str) -> None:
        ...

    @staticmethod
    @overload
    def write_line_if(condition: bool, value: typing.Any) -> None:
        ...

    @staticmethod
    @overload
    def write_line_if(condition: bool, value: typing.Any, category: str) -> None:
        ...

    @staticmethod
    @overload
    def write_line_if(condition: bool, message: str) -> None:
        ...

    @staticmethod
    @overload
    def write_line_if(condition: bool, message: System.Diagnostics.Debug.WriteIfInterpolatedStringHandler) -> None:
        ...

    @staticmethod
    @overload
    def write_line_if(condition: bool, message: str, category: str) -> None:
        ...

    @staticmethod
    @overload
    def write_line_if(condition: bool, message: System.Diagnostics.Debug.WriteIfInterpolatedStringHandler, category: str) -> None:
        ...


class Stopwatch(System.Object):
    """This class has no documentation."""

    FREQUENCY: int = ...

    IS_HIGH_RESOLUTION: bool = True

    @property
    def is_running(self) -> bool:
        ...

    @property
    def elapsed(self) -> datetime.timedelta:
        ...

    @property
    def elapsed_milliseconds(self) -> int:
        ...

    @property
    def elapsed_ticks(self) -> int:
        ...

    def __init__(self) -> None:
        ...

    @staticmethod
    @overload
    def get_elapsed_time(starting_timestamp: int) -> datetime.timedelta:
        ...

    @staticmethod
    @overload
    def get_elapsed_time(starting_timestamp: int, ending_timestamp: int) -> datetime.timedelta:
        ...

    @staticmethod
    def get_timestamp() -> int:
        ...

    def reset(self) -> None:
        ...

    def restart(self) -> None:
        ...

    def start(self) -> None:
        ...

    @staticmethod
    def start_new() -> System.Diagnostics.Stopwatch:
        ...

    def stop(self) -> None:
        ...

    def to_string(self) -> str:
        ...


class DebuggerBrowsableState(IntEnum):
    """This class has no documentation."""

    NEVER = 0

    COLLAPSED = 2

    ROOT_HIDDEN = 3


class DebuggerBrowsableAttribute(System.Attribute):
    """This class has no documentation."""

    @property
    def state(self) -> System.Diagnostics.DebuggerBrowsableState:
        ...

    def __init__(self, state: System.Diagnostics.DebuggerBrowsableState) -> None:
        ...


class DebuggerStepThroughAttribute(System.Attribute):
    """This class has no documentation."""

    def __init__(self) -> None:
        ...


class StackTrace(System.Object):
    """This class has no documentation."""

    METHODS_TO_SKIP: int = 0

    @property
    def frame_count(self) -> int:
        ...

    @overload
    def __init__(self) -> None:
        ...

    @overload
    def __init__(self, f_need_file_info: bool) -> None:
        ...

    @overload
    def __init__(self, skip_frames: int) -> None:
        ...

    @overload
    def __init__(self, skip_frames: int, f_need_file_info: bool) -> None:
        ...

    @overload
    def __init__(self, e: System.Exception) -> None:
        ...

    @overload
    def __init__(self, e: System.Exception, f_need_file_info: bool) -> None:
        ...

    @overload
    def __init__(self, e: System.Exception, skip_frames: int) -> None:
        ...

    @overload
    def __init__(self, e: System.Exception, skip_frames: int, f_need_file_info: bool) -> None:
        ...

    @overload
    def __init__(self, frame: System.Diagnostics.StackFrame) -> None:
        ...

    @overload
    def __init__(self, frames: System.Collections.Generic.IEnumerable[System.Diagnostics.StackFrame]) -> None:
        ...

    def get_frame(self, index: int) -> System.Diagnostics.StackFrame:
        ...

    def get_frames(self) -> typing.List[System.Diagnostics.StackFrame]:
        ...

    def to_string(self) -> str:
        ...


class ConditionalAttribute(System.Attribute):
    """This class has no documentation."""

    @property
    def condition_string(self) -> str:
        ...

    def __init__(self, condition_string: str) -> None:
        ...


class StackFrameExtensions(System.Object):
    """This class has no documentation."""

    @staticmethod
    def get_native_image_base(stack_frame: System.Diagnostics.StackFrame) -> System.IntPtr:
        ...

    @staticmethod
    def get_native_ip(stack_frame: System.Diagnostics.StackFrame) -> System.IntPtr:
        ...

    @staticmethod
    def has_il_offset(stack_frame: System.Diagnostics.StackFrame) -> bool:
        ...

    @staticmethod
    def has_method(stack_frame: System.Diagnostics.StackFrame) -> bool:
        ...

    @staticmethod
    def has_native_image(stack_frame: System.Diagnostics.StackFrame) -> bool:
        ...

    @staticmethod
    def has_source(stack_frame: System.Diagnostics.StackFrame) -> bool:
        ...


class DebuggerDisplayAttribute(System.Attribute):
    """This class has no documentation."""

    @property
    def value(self) -> str:
        ...

    @property
    def name(self) -> str:
        ...

    @name.setter
    def name(self, value: str) -> None:
        ...

    @property
    def type(self) -> str:
        ...

    @type.setter
    def type(self, value: str) -> None:
        ...

    @property
    def target(self) -> typing.Type:
        ...

    @target.setter
    def target(self, value: typing.Type) -> None:
        ...

    @property
    def target_type_name(self) -> str:
        ...

    @target_type_name.setter
    def target_type_name(self, value: str) -> None:
        ...

    def __init__(self, value: str) -> None:
        ...


class UnreachableException(System.Exception):
    """This class has no documentation."""

    @overload
    def __init__(self) -> None:
        ...

    @overload
    def __init__(self, message: str) -> None:
        ...

    @overload
    def __init__(self, message: str, inner_exception: System.Exception) -> None:
        ...


class StackTraceHiddenAttribute(System.Attribute):
    """This class has no documentation."""

    def __init__(self) -> None:
        ...


class Debugger(System.Object):
    """This class has no documentation."""

    DEFAULT_CATEGORY: str

    @staticmethod
    def break_for_user_unhandled_exception(exception: System.Exception) -> None:
        ...


class DebuggerStepperBoundaryAttribute(System.Attribute):
    """This class has no documentation."""

    def __init__(self) -> None:
        ...


class DebuggerTypeProxyAttribute(System.Attribute):
    """This class has no documentation."""

    @property
    def proxy_type_name(self) -> str:
        ...

    @property
    def target(self) -> typing.Type:
        ...

    @target.setter
    def target(self, value: typing.Type) -> None:
        ...

    @property
    def target_type_name(self) -> str:
        ...

    @target_type_name.setter
    def target_type_name(self, value: str) -> None:
        ...

    @overload
    def __init__(self, type: typing.Type) -> None:
        ...

    @overload
    def __init__(self, type_name: str) -> None:
        ...


class DebuggerDisableUserUnhandledExceptionsAttribute(System.Attribute):
    """This class has no documentation."""


class DebuggerVisualizerAttribute(System.Attribute):
    """This class has no documentation."""

    @property
    def visualizer_object_source_type_name(self) -> str:
        ...

    @property
    def visualizer_type_name(self) -> str:
        ...

    @property
    def description(self) -> str:
        ...

    @description.setter
    def description(self, value: str) -> None:
        ...

    @property
    def target(self) -> typing.Type:
        ...

    @target.setter
    def target(self, value: typing.Type) -> None:
        ...

    @property
    def target_type_name(self) -> str:
        ...

    @target_type_name.setter
    def target_type_name(self, value: str) -> None:
        ...

    @overload
    def __init__(self, visualizer_type_name: str) -> None:
        ...

    @overload
    def __init__(self, visualizer_type_name: str, visualizer_object_source_type_name: str) -> None:
        ...

    @overload
    def __init__(self, visualizer_type_name: str, visualizer_object_source: typing.Type) -> None:
        ...

    @overload
    def __init__(self, visualizer: typing.Type) -> None:
        ...

    @overload
    def __init__(self, visualizer: typing.Type, visualizer_object_source: typing.Type) -> None:
        ...

    @overload
    def __init__(self, visualizer: typing.Type, visualizer_object_source_type_name: str) -> None:
        ...


class DebuggableAttribute(System.Attribute):
    """This class has no documentation."""

    class DebuggingModes(IntEnum):
        """This class has no documentation."""

        NONE = ...

        DEFAULT = ...

        DISABLE_OPTIMIZATIONS = ...

        IGNORE_SYMBOL_STORE_SEQUENCE_POINTS = ...

        ENABLE_EDIT_AND_CONTINUE = ...

    @property
    def is_jit_tracking_enabled(self) -> bool:
        ...

    @property
    def is_jit_optimizer_disabled(self) -> bool:
        ...

    @property
    def debugging_flags(self) -> System.Diagnostics.DebuggableAttribute.DebuggingModes:
        ...

    @overload
    def __init__(self, is_jit_tracking_enabled: bool, is_jit_optimizer_disabled: bool) -> None:
        ...

    @overload
    def __init__(self, modes: System.Diagnostics.DebuggableAttribute.DebuggingModes) -> None:
        ...


class DebuggerHiddenAttribute(System.Attribute):
    """This class has no documentation."""

    def __init__(self) -> None:
        ...


class DebuggerNonUserCodeAttribute(System.Attribute):
    """This class has no documentation."""

    def __init__(self) -> None:
        ...


